import { FormatEs5 } from "./format_es5.js";
export const DEFAULT_FORMAT = FormatEs5;
export const DEFAULT_FORMATS = [DEFAULT_FORMAT];
/** Returns the default format if no formats are given. */
export function getFormatWithFallback(format) {
    return format || DEFAULT_FORMAT;
}
/** Returns the default formats if no formats are given. */
export function getFormatsWithFallback(formats) {
    return formats || DEFAULT_FORMATS;
}
/** Given an array of format names, and an array of `IFormat`, returns an array of `IFormat` restricted to those with matching names. */
export function getFormatIntersection(formatNames, formats) {
    const intersection = [];
    for (const f of formats) {
        if (formatNames.includes(f.id)) {
            intersection.push(f);
        }
    }
    return intersection;
}
/** Returns an object with format names as keys, and corresponding `IFormat` as values. */
export function getFormatLookup(formats) {
    const f = formats || DEFAULT_FORMATS;
    const formatLookup = {};
    for (const format of f) {
        formatLookup[format.id] = format;
    }
    return formatLookup;
}
