import { join } from "node:path";
import { existsSync, mkdirSync, } from "fs";
import { AttachmentDriverFilesystem } from "./attachment_drivers/filesystem.node.js";
import { DocDriverSqlite } from "./doc_drivers/sqlite.node.js";
/** A replica driver which persists data to the filesystem. */
export class ReplicaDriverFs {
    constructor(shareAddress, dirPath) {
        Object.defineProperty(this, "docDriver", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "attachmentDriver", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        if (!existsSync(dirPath)) {
            mkdirSync(dirPath);
        }
        this.docDriver = new DocDriverSqlite({
            filename: join(dirPath, `${shareAddress}.sql`),
            mode: "create-or-open",
            share: shareAddress,
        });
        this.attachmentDriver = new AttachmentDriverFilesystem(join(dirPath, "attachments"));
    }
}
