import { MultiformatReplica } from "./multiformat_replica.js";
/**
 * A replica holding a share's data, used to read, write, and synchronise data to.
 *
 * Should be closed using the `close` method when no longer being used.
 *
 * ```ts
 * const gardeningKeypair = await Crypto.generateShareKeypair("gardening");
 *
 * const myReplica = new Replica({
 *  driver: new ReplicaDriverMemory(gardeningKeypair.shareAddress),
 *  shareSecret: gardeningKeypair.secret
 * });
 * ```
 */
export class Replica extends MultiformatReplica {
    constructor(opts) {
        super({
            driver: opts.driver,
            config: {
                "es.5": {
                    shareSecret: opts.shareSecret,
                },
            },
        });
    }
}
