import * as fs from "node:fs/promises";
/** A server extension for populating a server with known shares. Use this to specify which shares you'd like your server to sync with others.
 *
 * You most likely want to pass this as the first extension to your server.
 */
export class ExtensionKnownShares {
    constructor(opts) {
        Object.defineProperty(this, "peer", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: null
        });
        Object.defineProperty(this, "knownSharesPath", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "onCreateReplica", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.knownSharesPath = opts.knownSharesPath;
        this.onCreateReplica = opts.onCreateReplica;
    }
    async register(peer) {
        this.peer = peer;
        const knownSharesRaw = await fs.readFile(this.knownSharesPath, "utf-8");
        const knownShares = JSON.parse(knownSharesRaw);
        for (const shareAddress of knownShares) {
            const replica = this.onCreateReplica(shareAddress);
            await this.peer.addReplica(replica);
        }
    }
    handler() {
        return Promise.resolve(null);
    }
}
