import * as dntShim from "../../_dnt.shims.js";
import { Peer } from "../peer/peer.js";
/** The core server logic. Combine this with a HTTP framework to create a fully-fledged server. */
export class ServerCore {
    /**
     * Create a new server with an array of extensions.
     * @param extensions - The extensions used by the server. Extensions will be registered in the order you provide them in, as one extension may depend on the actions of another. For example, the `ExtensionServeContent` may rely on a replica created by `ExtensionKnownShares`.
     */
    constructor(extensions) {
        Object.defineProperty(this, "extensions", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "peer", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "isReady", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.peer = new Peer();
        this.extensions = extensions;
        this.isReady = this.registerExtensions();
        console.log("Your server is running.");
    }
    async registerExtensions() {
        // Extensions must be registered sequentially, one-by-one,
        // As later extensions may depend on the actions of previous ones
        // e.g. The serve content extension using a share added by the known shares extension
        for (const extension of this.extensions) {
            await extension.register(this.peer);
        }
    }
    async handler(req) {
        await this.isReady;
        for (const extension of this.extensions) {
            const response = await extension.handler(req);
            if (response) {
                return response;
            }
        }
        return new dntShim.Response("Not found", { status: 404 });
    }
}
