import { FingerprintTree, xxhash64 } from "../../deps.js";
/** Derive an order from two document thumbnails. */
function compareThumbnails(a, b) {
    const [timestampA, hashA] = a.split(" ");
    const [timestampB, hashB] = b.split(" ");
    const timestampAInt = parseInt(timestampA);
    const timestampBInt = parseInt(timestampB);
    if (timestampAInt > timestampBInt) {
        return 1;
    }
    else if (timestampAInt < timestampBInt) {
        return -1;
    }
    if (hashA > hashB) {
        return 1;
    }
    else if (hashA < hashB) {
        return -1;
    }
    return 0;
}
/** A lifting monoid which hashes a document thumbnail, and combines them using addition. */
const docThumbnailMonoid = {
    lift: (v) => {
        return xxhash64(v);
    },
    combine: (a, b) => {
        return a + b;
    },
    neutral: BigInt(0),
};
/** A FingerprintTree preconfigured for the insertion of DocThumbnails. */
export class DocThumbnailTree extends FingerprintTree {
    constructor() {
        super(docThumbnailMonoid, compareThumbnails);
    }
}
