import { RangeMessenger } from "../../deps.js";
import { bigIntFromHex, bigIntToHex } from "../util/bigint.js";
const TYPE_MAPPINGS = {
    "EMPTY_SET": "emptySet",
    "LOWER_BOUND": "lowerBound",
    "PAYLOAD": "payload",
    "EMPTY_PAYLOAD": "emptyPayload",
    "DONE": "done",
    "FINGERPRINT": "fingerprint",
    "TERMINAL": "terminal",
};
const encoding = {
    encode: {
        emptySet: (canRespond) => ({
            type: "EMPTY_SET",
            canRespond,
        }),
        lowerBound: (x) => ({
            type: "LOWER_BOUND",
            value: x,
        }),
        payload: (v, end) => ({
            type: "PAYLOAD",
            payload: v,
            ...(end ? { end } : {}),
        }),
        emptyPayload: (upperBound) => ({
            type: "EMPTY_PAYLOAD",
            upperBound,
        }),
        done: (y) => ({
            type: "DONE",
            upperBound: y,
        }),
        fingerprint: (fp, y) => ({
            type: "FINGERPRINT",
            fingerprint: bigIntToHex(fp),
            upperBound: y,
        }),
        terminal: () => ({
            type: "TERMINAL",
        }),
    },
    decode: {
        getType: (obj) => {
            return TYPE_MAPPINGS[obj.type];
        },
        emptySet: (obj) => {
            if (obj.type === "EMPTY_SET") {
                return obj.canRespond;
            }
            throw "Can't decode";
        },
        lowerBound: (obj) => {
            if (obj.type === "LOWER_BOUND") {
                return obj.value;
            }
            throw "Can't decode";
        },
        payload: (obj) => {
            if (obj.type === "PAYLOAD") {
                return {
                    value: obj.payload,
                    ...(obj.end ? { end: obj.end } : {}),
                };
            }
            throw "Can't decode";
        },
        emptyPayload: (obj) => {
            if (obj.type === "EMPTY_PAYLOAD") {
                return obj.upperBound;
            }
            throw "Can't decode";
        },
        done: (obj) => {
            if (obj.type === "DONE") {
                return obj.upperBound;
            }
            throw "Can't decode";
        },
        fingerprint: (obj) => {
            if (obj.type === "FINGERPRINT") {
                return {
                    fingerprint: bigIntFromHex(obj.fingerprint),
                    upperBound: obj.upperBound,
                };
            }
            throw "Can't decode";
        },
        terminal: (obj) => {
            if (obj.type === "TERMINAL") {
                return true;
            }
            throw "Can't decode";
        },
    },
};
export class EarthstarRangeMessenger extends RangeMessenger {
    constructor(tree, payloadThreshold, rangeDivision) {
        super({
            tree,
            encoding,
            fingerprintEquals: (a, b) => a === b,
            payloadThreshold,
            rangeDivision,
        });
    }
}
