import * as dntShim from "../../_dnt.shims.js";
import { deferred } from "../../deps.js";
import { base32BytesToString } from "../crypto/base32.js";
import { Crypto } from "../crypto/crypto.js";
export class AttachmentStreamInfo {
    constructor() {
        Object.defineProperty(this, "transformer", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "updatableHash", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: Crypto.updatableSha256()
        });
        Object.defineProperty(this, "size", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: deferred()
        });
        Object.defineProperty(this, "hash", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: deferred()
        });
        const { updatableHash, size, hash } = this;
        let currentSize = 0;
        this.transformer = new dntShim.TransformStream({
            transform(chunk, controller) {
                updatableHash.update(chunk);
                currentSize += chunk.byteLength;
                controller.enqueue(chunk);
            },
            flush() {
                const digest = updatableHash.digest();
                hash.resolve(base32BytesToString(digest));
                size.resolve(currentSize);
            },
        });
    }
    get writable() {
        return this.transformer.writable;
    }
    get readable() {
        return this.transformer.readable;
    }
}
