import * as dntShim from "../../_dnt.shims.js";
export function bytesToStream(bytes) {
    return new dntShim.ReadableStream({
        start(controller) {
            for (let i = 0; i <= bytes.length; i += 8) {
                controller.enqueue(bytes.slice(i, i + 8));
            }
            controller.close();
        },
    });
}
export async function streamToBytes(stream) {
    let bytes = new Uint8Array();
    await stream.pipeTo(new dntShim.WritableStream({
        write(chunk) {
            const nextBytes = new Uint8Array(bytes.length + chunk.length);
            nextBytes.set(bytes);
            nextBytes.set(chunk, bytes.length);
            bytes = nextBytes;
        },
    }));
    return bytes;
}
export async function getStreamSize(stream) {
    let size = 0;
    const sink = new dntShim.WritableStream({
        write(chunk) {
            size += chunk.byteLength;
        },
    });
    await stream.pipeTo(sink);
    return size;
}
export async function readStream(stream) {
    const arr = [];
    const writable = new dntShim.WritableStream({
        write(entry) {
            arr.push(entry);
        },
    });
    await stream.pipeTo(writable);
    return arr;
}
