"use strict";
/**
 * Tools for efficient reconciliation of sets. This is an implementation of range-based set reconciliation as described in Aljoscha Meyer's master thesis (https://github.com/AljoschaMeyer/master_thesis).
 *
 * At the broadest level:
 * 1. For each set you wish to reconcile, instantiate a FingerprintTree and insert that set's elements.
 * 2. Create a RangeMessenger for each FingerprintTree.
 * 3. Reconcile the sets by exchanging messages between two RangeMessengers.
 *
 * Reconciliation can be conducted locally or over a network.
 * This library does not have any opinion on message encoding or transport. Users provide these details themselves.
 * Currently the FingerprintTrees are only persisted in memory.
 *
 * @module
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
Object.defineProperty(exports, "__esModule", { value: true });
__exportStar(require("./src/fingerprint_tree/fingerprint_tree.js"), exports);
__exportStar(require("./src/range_messenger/range_messenger.js"), exports);
__exportStar(require("./src/range_messenger/range_messenger_config.js"), exports);
__exportStar(require("./src/lifting_monoid.js"), exports);
__exportStar(require("./src/util.js"), exports);
