"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.sizeMonoid = exports.concatMonoid = exports.combineMonoid = void 0;
/** Combine two lifting monoids into a new one. */
function combineMonoid(a, b) {
    return {
        lift: (i) => {
            return [a.lift(i), b.lift(i)];
        },
        combine: (ia, ib) => {
            const fst = a.combine(ia[0], ib[0]);
            const snd = b.combine(ia[1], ib[1]);
            return [fst, snd];
        },
        neutral: [a.neutral, b.neutral],
    };
}
exports.combineMonoid = combineMonoid;
/** A monoid which lifts the member as a string, and combines by concatenating together. */
exports.concatMonoid = {
    lift: (a) => a,
    combine: (a, b) => {
        const fst = a === "0" ? "" : a;
        const snd = b === "0" ? "" : b;
        return fst + snd;
    },
    neutral: "0",
};
/** A monoid which lifts the member as 1, and combines by adding together. */
exports.sizeMonoid = {
    lift: (_a) => 1,
    combine: (a, b) => a + b,
    neutral: 0,
};
