"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.pathChars = exports.pathPunctuation = exports.workspaceAddressChars = exports.workspaceKeyChars = exports.workspaceNameChars = exports.authorAddressChars = exports.authorKeyChars = exports.authorNameChars = exports.b32chars = exports.digits = exports.alphaUpper = exports.alphaLower = exports.isDigit = exports.isOnlyPrintableAscii = exports.onlyHasChars = void 0;
const bytes_js_1 = require("../util/bytes.js");
/** Check that a string only contains character from a string of allowed characters. */
function onlyHasChars(str, allowedChars) {
    for (let s of str) {
        if (allowedChars.indexOf(s) === -1)
            return false;
    }
    return true;
}
exports.onlyHasChars = onlyHasChars;
/** Check that a string contains only printable ASCII */
function isOnlyPrintableAscii(s) {
    let bytes = (0, bytes_js_1.stringToBytes)(s);
    for (let byte of bytes) {
        // char must be between ' ' (space) and '~' inclusive
        if (byte < 32 || byte > 126)
            return false;
    }
    return true;
}
exports.isOnlyPrintableAscii = isOnlyPrintableAscii;
/* Check that a string is exactly one digit. */
function isDigit(ch) {
    if (ch === "")
        return false;
    return exports.digits.indexOf(ch) !== -1;
}
exports.isDigit = isDigit;
/** Lowercase alphabetical characters. */
exports.alphaLower = "abcdefghijklmnopqrstuvwxyz";
/** Uppercase alphabetical characters. */
exports.alphaUpper = exports.alphaLower.toUpperCase();
/** All digits. */
exports.digits = "0123456789";
/** All characters allowed in base32. */
exports.b32chars = exports.alphaLower + "234567";
/** All characters allowed in an identity's short name. */
exports.authorNameChars = exports.alphaLower + exports.digits;
/** All characters allowed in an identity's pub key. */
exports.authorKeyChars = exports.b32chars;
/** All characters allowed in an identity's public address. */
exports.authorAddressChars = exports.authorNameChars + exports.b32chars + "@.";
/** All characters allowed in a share's name. */
exports.workspaceNameChars = exports.alphaLower + exports.digits;
/** All characters allowed in a share's key. */
exports.workspaceKeyChars = exports.b32chars;
/** All characters allowed in a share's address. */
exports.workspaceAddressChars = exports.workspaceNameChars + exports.b32chars + "+.";
// Characters allowed in Earthstar paths
//---------------------------------------------
// These allowed path characters should be usable in regular web URLs
// without percent-encoding them or interfering with the rest of the URL.
//
// Allowed      Earthstar Meaning
//    a-zA-Z0-9    no meaning
//    '()-_$&,:=   no meaning
//    /            starts paths; path segment separator
//    !            ephemeral docs must contain at least one '!'
//    ~            denotes path ownership (write permissions)
//    +@.          used by workspace and author names but allowed elsewhere too
//    %            used for percent-encoding
//
// Disallowed       Reason
//    space            not allowed in URLs
//    <>"[\]^`{|}      not allowed in URLs (though some browsers allow some of them)
//    ?                to avoid confusion with URL query parameters
//    #                to avoid confusion with URL anchors
//    ;                no reason
//    *                no reason; maybe useful for querying in the future
//    non-ASCII chars  to avoid trouble with Unicode normalization
//    ASCII whitespace
//    ASCII control characters
//
// (Regular URL character rules are in RFC3986, RFC1738, and https://url.spec.whatwg.org/#url-code-points )
//
// To use other characters in a path, percent-encode them using encodeURI.
// For example
//      desiredPath = '/food/🍆/nutrition'
//      earthstarPath = encodeURI(desiredPath);  // --> '/food/%F0%9F%8D%86/nutrition'
//      store it as earthstarPath
//      for display to users again, run decodeURI(earthstarPath)
//
/** All special characters permitted in a document's path. */
exports.pathPunctuation = "/'()-._~!$&+,:=@%"; // note double quotes are not included
/** All characters permitted in a document's path. */
exports.pathChars = exports.alphaLower + exports.alphaUpper + exports.digits + exports.pathPunctuation;
