"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CryptoDriverNoble = void 0;
const bytes_js_1 = require("../util/bytes.js");
const deps_js_1 = require("../../deps.js");
const { createHash } = deps_js_1.sha256_uint8array;
//--------------------------------------------------
const log_js_1 = require("../util/log.js");
const updatable_hash_js_1 = require("./updatable_hash.js");
const logger = new log_js_1.Logger("crypto-driver-noble", "cyan");
//================================================================================
/**
 * A version of the ICryptoDriver interface backed by noble/ed25519.
 * The slowest crypto driver available, but works everywhere.
 */
const CryptoDriverNoble = class {
    static sha256(input) {
        if (typeof input === "string") {
            return Promise.resolve(createHash("sha256").update(input, "utf-8").digest());
        }
        else {
            return Promise.resolve(createHash("sha256").update(input).digest());
        }
    }
    static updatableSha256() {
        return new updatable_hash_js_1.UpdatableHash({
            hash: createHash("sha256"),
            update: (hash, data) => hash.update(data),
            digest: (hash) => hash.digest(),
        });
    }
    static async generateKeypairBytes() {
        logger.debug("generateKeypairBytes");
        const secret = deps_js_1.ed.utils.randomPrivateKey();
        const pubkey = await deps_js_1.ed.getPublicKey(secret);
        return {
            pubkey,
            secret,
        };
    }
    static sign(keypairBytes, msg) {
        logger.debug("sign");
        if (typeof msg === "string")
            msg = (0, bytes_js_1.stringToBytes)(msg);
        return deps_js_1.ed.sign(msg, keypairBytes.secret);
    }
    static async verify(publicKey, sig, msg) {
        logger.debug("verify");
        try {
            if (typeof msg === "string")
                msg = (0, bytes_js_1.stringToBytes)(msg);
            const result = await deps_js_1.ed.verify(sig, msg, publicKey);
            return result;
        }
        catch {
            return false;
        }
    }
};
exports.CryptoDriverNoble = CryptoDriverNoble;
