"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isAuthorKeypair = exports.decodeKeypairToBytes = exports.encodeShareKeypairToStrings = exports.encodeAuthorKeypairToStrings = void 0;
const errors_js_1 = require("../util/errors.js");
const base32_js_1 = require("./base32.js");
const addresses_js_1 = require("../core-validators/addresses.js");
//================================================================================
/** Combine a shortname with a raw KeypairBytes to make an AuthorKeypair */
function encodeAuthorKeypairToStrings(shortname, pair) {
    return ({
        address: (0, addresses_js_1.assembleAuthorAddress)(shortname, (0, base32_js_1.base32BytesToString)(pair.pubkey)),
        secret: (0, base32_js_1.base32BytesToString)(pair.secret),
    });
}
exports.encodeAuthorKeypairToStrings = encodeAuthorKeypairToStrings;
/** Combine a name with a raw KeypairBytes to make an ShareKeypair */
function encodeShareKeypairToStrings(name, pair) {
    return ({
        address: (0, addresses_js_1.assembleShareAddress)(name, (0, base32_js_1.base32BytesToString)(pair.pubkey)),
        secret: (0, base32_js_1.base32BytesToString)(pair.secret),
    });
}
exports.encodeShareKeypairToStrings = encodeShareKeypairToStrings;
/** Convert a keypair (author / share) back into a raw KeypairBytes for use in crypto operations. */
function decodeKeypairToBytes(pair) {
    try {
        const address = isAuthorKeypair(pair) ? pair.address : pair.shareAddress;
        const parsed = (0, addresses_js_1.parseAuthorOrShareAddress)(address);
        if ((0, errors_js_1.isErr)(parsed))
            return parsed;
        const bytes = {
            pubkey: (0, base32_js_1.base32StringToBytes)(parsed.pubkey),
            secret: (0, base32_js_1.base32StringToBytes)(pair.secret),
        };
        if (bytes.pubkey.length !== 32) {
            // this is already checked by parseAuthorAddress so we can't test it here
            // but we'll test it again just to make sure.
            return new errors_js_1.ValidationError(`pubkey bytes should be 32 bytes long, not ${bytes.pubkey.length} after base32 decoding.  ${address}`);
        }
        if (bytes.secret.length !== 32) {
            return new errors_js_1.ValidationError(`secret bytes should be 32 bytes long, not ${bytes.secret.length} after base32 decoding.  ${pair.secret}`);
        }
        return bytes;
    }
    catch (err) {
        return new errors_js_1.ValidationError("crash while decoding author keypair: " + err.message);
    }
}
exports.decodeKeypairToBytes = decodeKeypairToBytes;
function isAuthorKeypair(keypair) {
    return "address" in keypair;
}
exports.isAuthorKeypair = isAuthorKeypair;
