"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AttachmentDriverFilesystem = void 0;
const dntShim = __importStar(require("../../../_dnt.shims.js"));
const errors_js_1 = require("../../util/errors.js");
const misc_js_1 = require("../../util/misc.js");
const crypto_js_1 = require("../../crypto/crypto.js");
const attachment_stream_info_js_1 = require("../../util/attachment_stream_info.js");
const fs_walk_1 = require("@nodelib/fs.walk");
const fs = __importStar(require("node:fs/promises"));
const path = __importStar(require("node:path"));
const buffers_js_1 = require("../../util/buffers.js");
const deps_js_1 = require("../../../deps.js");
/** An attachment driver which persists attachments using the local filesystem.
 * Works with Node.
 */
class AttachmentDriverFilesystem {
    /** @param path - The filesystem path all attachments will be stored under. */
    constructor(path) {
        Object.defineProperty(this, "path", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "closed", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: false
        });
        this.path = path;
    }
    async ensurePath(...args) {
        try {
            await fs.lstat(path.join(this.path, ...args));
        }
        catch {
            await fs.mkdir(path.join(this.path, ...args), {
                recursive: true,
            });
        }
    }
    async stage(formatName, attachment) {
        if (this.closed)
            throw new errors_js_1.ReplicaIsClosedError();
        // Create the path
        await this.ensurePath("staging", formatName);
        const tempKey = (0, misc_js_1.randomId)();
        const stagingPath = path.join(this.path, "staging", formatName, tempKey);
        if (attachment instanceof Uint8Array) {
            await fs.writeFile(stagingPath, attachment);
            const hash = await crypto_js_1.Crypto.sha256base32(attachment);
            return {
                hash,
                size: attachment.byteLength,
                commit: async () => {
                    try {
                        await fs.lstat(path.join(this.path, formatName));
                    }
                    catch {
                        await fs.mkdir(path.join(this.path, formatName));
                    }
                    return fs.rename(stagingPath, path.join(this.path, formatName, hash));
                },
                reject: () => {
                    return fs.rm(stagingPath);
                },
            };
        }
        const attachmentStreamInfo = new attachment_stream_info_js_1.AttachmentStreamInfo();
        try {
            await fs.truncate(stagingPath);
        }
        catch {
            // It's fine.
        }
        const file = await fs.open(stagingPath, "w");
        const writeStream = file.createWriteStream({ start: 0 });
        await attachment.pipeThrough(attachmentStreamInfo).pipeTo(new dntShim.WritableStream({
            async write(chunk) {
                await new Promise((res) => {
                    writeStream.write(chunk, "binary", res);
                });
            },
            close() {
                writeStream.end();
            },
        }));
        const hash = await attachmentStreamInfo.hash;
        const size = await attachmentStreamInfo.size;
        return {
            hash,
            size,
            commit: () => {
                return fs.rename(stagingPath, path.join(this.path, formatName, hash));
            },
            reject: () => {
                return fs.rm(stagingPath);
            },
        };
    }
    async erase(formatName, attachmentHash) {
        if (this.closed)
            throw new errors_js_1.ReplicaIsClosedError();
        const filePath = path.join(this.path, formatName, attachmentHash);
        try {
            await fs.rm(filePath);
            return true;
        }
        catch {
            return new errors_js_1.ValidationError(`Attachment not found`);
        }
    }
    async wipe() {
        if (this.closed)
            throw new errors_js_1.ReplicaIsClosedError();
        await this.clearStaging();
        try {
            const entries = await fs.readdir(this.path);
            for (const entry of entries) {
                await fs.rm(path.join(this.path, entry), { recursive: true });
            }
        }
        catch {
            // Do nothing...
            return Promise.resolve();
        }
    }
    async getAttachment(formatName, attachmentHash) {
        if (this.closed)
            throw new errors_js_1.ReplicaIsClosedError();
        const filePath = path.join(this.path, formatName, attachmentHash);
        try {
            await fs.lstat(filePath);
        }
        catch {
            return undefined;
        }
        return {
            bytes: async () => (0, buffers_js_1.bufferToBytes)(await fs.readFile(filePath)),
            stream: () => {
                const readableStream = new dntShim.ReadableStream({
                    async start(controller) {
                        const file = await fs.open(filePath, "r");
                        const readStream = file.createReadStream();
                        readStream.on("end", () => {
                            controller.close();
                        });
                        readStream.on("data", (chunk) => {
                            controller.enqueue((0, buffers_js_1.bufferToBytes)(chunk));
                        });
                    },
                });
                return Promise.resolve(readableStream);
            },
        };
    }
    async clearStaging() {
        if (this.closed)
            throw new errors_js_1.ReplicaIsClosedError();
        try {
            await fs.rm(path.join(this.path, "staging"), { recursive: true });
        }
        catch {
            return Promise.resolve();
        }
    }
    async filter(attachments) {
        if (this.closed)
            throw new errors_js_1.ReplicaIsClosedError();
        try {
            await fs.lstat(this.path);
        }
        catch {
            return [];
        }
        const erased = [];
        const walkDeferred = (0, deps_js_1.deferred)();
        (0, fs_walk_1.walk)(this.path, async (_error, entries) => {
            for (const entry of entries) {
                if (entry.dirent.isFile()) {
                    const format = path.dirname(path.relative(this.path, entry.path));
                    const hash = path.basename(entry.path);
                    if (format !== "staging") {
                        const allowedHashes = attachments[format];
                        if (allowedHashes && !allowedHashes.has(hash)) {
                            const res = await this.erase(format, hash);
                            if (!(0, errors_js_1.isErr)(res)) {
                                erased.push({ format, hash });
                            }
                        }
                    }
                }
            }
            walkDeferred.resolve();
        });
        await walkDeferred;
        return erased;
    }
    isClosed() {
        return this.closed;
    }
    async close(erase) {
        if (this.closed)
            throw new errors_js_1.ReplicaIsClosedError();
        if (erase) {
            await this.wipe();
        }
        this.closed = true;
        return;
    }
}
exports.AttachmentDriverFilesystem = AttachmentDriverFilesystem;
