"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AttachmentDriverIndexedDB = void 0;
const dntShim = __importStar(require("../../../_dnt.shims.js"));
const log_js_1 = require("../../util/log.js");
const errors_js_1 = require("../../util/errors.js");
const crypto_js_1 = require("../../crypto/crypto.js");
const misc_js_1 = require("../../util/misc.js");
const streams_js_1 = require("../../util/streams.js");
const deps_js_1 = require("../../../deps.js");
const logger = new log_js_1.Logger("replica driver indexeddb", "gold");
const ATTACHMENT_STAGING_STORE = "attachment_staging_index";
const ATTACHMENT_INDEX_STORE = "attachments_index";
const ATTACHMENT_BYTES_STORE = "attachments_bytes";
class AttachmentDriverIndexedDB {
    constructor(share, namespace) {
        Object.defineProperty(this, "db", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: (0, deps_js_1.deferred)()
        });
        Object.defineProperty(this, "share", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "closed", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: false
        });
        this.share = share;
        // dnt-shim-ignore
        if (!window.indexedDB) {
            throw new errors_js_1.EarthstarError("IndexedDB is not supported by this runtime.");
        }
        const request = dntShim.dntGlobalThis.indexedDB.open(`earthstar:share_attachments:${this.share}${namespace ? `/${namespace}` : ""}`, 1);
        request.onerror = () => {
            logger.error(`Could not open IndexedDB for ${this.share}'s attachments.`);
            logger.error(request.error);
            throw new errors_js_1.EarthstarError(`Could not open IndexedDB for ${this.share}'s attachments.`);
        };
        request.onupgradeneeded = function () {
            const db = request.result;
            db.createObjectStore(ATTACHMENT_BYTES_STORE);
            db.createObjectStore(ATTACHMENT_INDEX_STORE, { keyPath: "id" });
            db.createObjectStore(ATTACHMENT_STAGING_STORE, { keyPath: "id" });
        };
        request.onsuccess = () => {
            this.db.resolve(request.result);
        };
    }
    getIndexKey(formatName, attachmentHash) {
        return `${formatName}___${attachmentHash}`;
    }
    async getAttachment(formatName, attachmentHash) {
        if (this.closed)
            throw new errors_js_1.ReplicaIsClosedError();
        const resultDeferred = (0, deps_js_1.deferred)();
        const indexKey = this.getIndexKey(formatName, attachmentHash);
        const db = await this.db;
        const transaction = db.transaction([
            ATTACHMENT_INDEX_STORE,
        ], "readonly");
        const getKey = transaction.objectStore(ATTACHMENT_INDEX_STORE).get(indexKey);
        const blobKeyDeferred = (0, deps_js_1.deferred)();
        getKey.onerror = () => {
            blobKeyDeferred.reject();
        };
        getKey.onsuccess = () => {
            if (getKey.result === undefined) {
                blobKeyDeferred.reject();
            }
            else {
                blobKeyDeferred.resolve(getKey.result.blobKey);
            }
        };
        blobKeyDeferred.then((blobKey) => {
            const blobTransaction = db.transaction([ATTACHMENT_BYTES_STORE], "readonly");
            const getBlob = blobTransaction.objectStore(ATTACHMENT_BYTES_STORE).get(blobKey);
            getBlob.onsuccess = () => {
                const blob = new dntShim.Blob([getBlob.result]);
                resultDeferred.resolve({
                    bytes: async () => new Uint8Array(await blob.arrayBuffer()),
                    stream: () => Promise.resolve(
                    // Need to do this for Node's sake.
                    blob.stream()),
                });
            };
            getBlob.onerror = () => {
                // should probably delete the index here...
                resultDeferred.resolve(undefined);
            };
        }).catch(() => {
            resultDeferred.resolve(undefined);
        });
        return resultDeferred;
    }
    async erase(formatName, attachmentHash) {
        if (this.closed)
            throw new errors_js_1.ReplicaIsClosedError();
        const resultDeferred = (0, deps_js_1.deferred)();
        const indexKey = this.getIndexKey(formatName, attachmentHash);
        const db = await this.db;
        const transaction = db.transaction([
            ATTACHMENT_INDEX_STORE,
        ], "readonly");
        const getKey = transaction.objectStore(ATTACHMENT_INDEX_STORE).get(indexKey);
        const blobKeyDeferred = (0, deps_js_1.deferred)();
        getKey.onerror = () => {
            blobKeyDeferred.reject();
        };
        getKey.onsuccess = () => {
            db.transaction([ATTACHMENT_INDEX_STORE], "readwrite").objectStore(ATTACHMENT_INDEX_STORE).delete(indexKey);
            if (getKey.result === undefined) {
                blobKeyDeferred.reject();
            }
            else {
                blobKeyDeferred.resolve(getKey.result.blobKey);
            }
        };
        blobKeyDeferred.then((blobKey) => {
            const deleteBlob = db.transaction([ATTACHMENT_BYTES_STORE], "readwrite")
                .objectStore(ATTACHMENT_BYTES_STORE).delete(blobKey);
            deleteBlob.onsuccess = () => {
                resultDeferred.resolve(true);
            };
            deleteBlob.onerror = () => {
                // should probably delete the index here...
                resultDeferred.resolve(undefined);
            };
        }).catch(() => {
            resultDeferred.resolve(new errors_js_1.ValidationError("No attachment found"));
        });
        return resultDeferred;
    }
    async stage(formatName, attachment) {
        if (this.closed)
            throw new errors_js_1.ReplicaIsClosedError();
        // How this works:
        // We write the data to a single IndexedDB object store which contains both staged / committed attachments.
        // When staging, we write a row to a store of staged attachments with the attachment's key.
        // If the attachment is reject, both the staging record and the attachment bytes are deleted.
        // If the attachment is committed, this record is moved from the store of staging records to the store of committed records.
        // The motivation is that we only write data once, as we have to load it all into memory first.
        const db = await this.db;
        const bytes = attachment instanceof Uint8Array
            ? attachment
            : await (0, streams_js_1.streamToBytes)(attachment);
        const hash = await crypto_js_1.Crypto.sha256base32(bytes);
        const indexKey = this.getIndexKey(formatName, hash);
        const blobKey = `${indexKey}___${(0, misc_js_1.randomId)()}`;
        const transaction = db.transaction([
            ATTACHMENT_BYTES_STORE,
            ATTACHMENT_STAGING_STORE,
        ], "readwrite");
        const dataPut = transaction.objectStore(ATTACHMENT_BYTES_STORE).put(bytes, blobKey);
        const stagingPut = transaction.objectStore(ATTACHMENT_STAGING_STORE).put({
            id: indexKey,
            blobKey,
        });
        const putDeferred = (0, deps_js_1.deferred)();
        const stagingDeferred = (0, deps_js_1.deferred)();
        dataPut.onsuccess = () => putDeferred.resolve();
        stagingPut.onsuccess = () => stagingDeferred.resolve();
        await putDeferred;
        await stagingDeferred;
        return {
            hash,
            size: bytes.byteLength,
            reject: async () => {
                const deleteTransaction = db.transaction([
                    ATTACHMENT_BYTES_STORE,
                    ATTACHMENT_STAGING_STORE,
                ], "readwrite");
                const dataDelete = deleteTransaction.objectStore(ATTACHMENT_BYTES_STORE).delete(blobKey);
                const stagingDelete = deleteTransaction.objectStore(ATTACHMENT_STAGING_STORE).delete(indexKey);
                const deleteDeferred = (0, deps_js_1.deferred)();
                const stagingDeleteDeferred = (0, deps_js_1.deferred)();
                dataDelete.onsuccess = () => {
                    deleteDeferred.resolve();
                };
                stagingDelete.onsuccess = () => {
                    stagingDeleteDeferred.resolve();
                };
                await Promise.all([deleteDeferred, stagingDeleteDeferred]);
            },
            commit: async () => {
                // delete staging index
                // write proper index
                const transaction = db.transaction([
                    ATTACHMENT_INDEX_STORE,
                    ATTACHMENT_STAGING_STORE,
                ], "readwrite");
                const deleteStaging = transaction.objectStore(ATTACHMENT_STAGING_STORE).delete(indexKey);
                const putRealIndex = transaction.objectStore(ATTACHMENT_INDEX_STORE)
                    .put({ id: indexKey, blobKey });
                const deleteDeferred = (0, deps_js_1.deferred)();
                const realIndexDeferred = (0, deps_js_1.deferred)();
                deleteStaging.onsuccess = () => deleteDeferred.resolve();
                putRealIndex.onsuccess = () => realIndexDeferred.resolve();
                await Promise.all([deleteDeferred, realIndexDeferred]);
            },
        };
    }
    async wipe() {
        if (this.closed)
            throw new errors_js_1.ReplicaIsClosedError();
        const db = await this.db;
        const transaction = db.transaction([
            ATTACHMENT_BYTES_STORE,
            ATTACHMENT_INDEX_STORE,
            ATTACHMENT_STAGING_STORE,
        ], "readwrite");
        const wipeData = transaction.objectStore(ATTACHMENT_BYTES_STORE).clear();
        const wipeIndex = transaction.objectStore(ATTACHMENT_INDEX_STORE).clear();
        const wipeStaging = transaction.objectStore(ATTACHMENT_STAGING_STORE)
            .clear();
        const dataDeferred = (0, deps_js_1.deferred)();
        const indexDeferred = (0, deps_js_1.deferred)();
        const stagingDeferred = (0, deps_js_1.deferred)();
        wipeData.onsuccess = () => dataDeferred.resolve();
        wipeIndex.onsuccess = () => indexDeferred.resolve();
        wipeStaging.onsuccess = () => stagingDeferred.resolve();
        await Promise.all([dataDeferred, indexDeferred, stagingDeferred]);
    }
    async clearStaging() {
        if (this.closed)
            throw new errors_js_1.ReplicaIsClosedError();
        // iterate through all the staging indexes and delete...
        const db = await this.db;
        const transaction = db.transaction([
            ATTACHMENT_BYTES_STORE,
            ATTACHMENT_STAGING_STORE,
        ]);
        const cursorReq = transaction.objectStore(ATTACHMENT_STAGING_STORE)
            .openCursor();
        const result = (0, deps_js_1.deferred)();
        cursorReq.onsuccess = () => {
            const res = cursorReq.result;
            if (!res) {
                // done
                result.resolve();
                return;
            }
            const blobKey = res.value.blobKey;
            const deleteBlob = transaction.objectStore(ATTACHMENT_BYTES_STORE).delete(blobKey);
            res.delete();
            deleteBlob.onsuccess = () => {
                res.continue();
            };
            deleteBlob.onerror = () => {
                res.continue();
            };
        };
        return result;
    }
    async filter(attachments) {
        if (this.closed)
            throw new errors_js_1.ReplicaIsClosedError();
        const db = await this.db;
        const transaction = db.transaction([
            ATTACHMENT_BYTES_STORE,
            ATTACHMENT_INDEX_STORE,
        ], "readwrite");
        // TODO: There is something going wrong with the batching in tests, but I think it's to do with the shim we're using.
        // In a browser I'm confident this should work correctly — if not, the culprit is here!
        const BATCH_SIZE = 50;
        const deleted = [];
        const deletionOps = [];
        const wipeDeferred = (0, deps_js_1.deferred)();
        const wipeBatch = (range = null) => {
            const getAllReq = transaction.objectStore(ATTACHMENT_INDEX_STORE)
                .getAll(range, BATCH_SIZE);
            getAllReq.onsuccess = () => {
                const results = getAllReq.result;
                for (const result of results) {
                    const [format, hash] = result.id.split("___");
                    if (attachments[format] && !attachments[format].has(hash)) {
                        deleted.push({ format, hash });
                        deletionOps.push(this.erase(format, hash));
                    }
                }
                if (results.length === BATCH_SIZE) {
                    const lastItem = results[results.length - 1];
                    const nextKeyRange = IDBKeyRange.lowerBound(lastItem.id, true);
                    wipeBatch(nextKeyRange);
                }
                else {
                    wipeDeferred.resolve();
                }
            };
        };
        wipeBatch();
        await wipeDeferred;
        await Promise.all(deletionOps);
        return deleted;
    }
    isClosed() {
        return this.closed;
    }
    async close(erase) {
        if (this.closed)
            throw new errors_js_1.ReplicaIsClosedError();
        if (erase) {
            await this.wipe();
        }
        this.closed = true;
        const db = await this.db;
        db.close();
        await (0, misc_js_1.sleep)(20);
        return;
    }
}
exports.AttachmentDriverIndexedDB = AttachmentDriverIndexedDB;
