"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AttachmentDriverMemory = void 0;
const dntShim = __importStar(require("../../../_dnt.shims.js"));
const crypto_js_1 = require("../../crypto/crypto.js");
const errors_js_1 = require("../../util/errors.js");
const streams_js_1 = require("../../util/streams.js");
/** An attachment driver which persists attachments in memory.
 * Works everywhere.
 */
class AttachmentDriverMemory {
    constructor() {
        Object.defineProperty(this, "stagingMap", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Map()
        });
        Object.defineProperty(this, "attachmentMap", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Map()
        });
        Object.defineProperty(this, "closed", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: false
        });
    }
    getKey(formatName, attachmentHash) {
        return `${formatName}___${attachmentHash}`;
    }
    getAttachment(formatName, attachmentHash) {
        if (this.closed)
            throw new errors_js_1.ReplicaIsClosedError();
        const key = this.getKey(formatName, attachmentHash);
        const attachment = this.attachmentMap.get(key);
        if (!attachment) {
            return Promise.resolve(undefined);
        }
        return Promise.resolve({
            bytes: async () => new Uint8Array(await attachment.arrayBuffer()),
            stream: () => Promise.resolve(
            // Need to do this for Node's sake.
            attachment.stream()),
        });
    }
    async stage(formatName, attachment) {
        if (this.closed)
            throw new errors_js_1.ReplicaIsClosedError();
        const bytes = attachment instanceof Uint8Array
            ? attachment
            : await (0, streams_js_1.streamToBytes)(attachment);
        const hash = await crypto_js_1.Crypto.sha256base32(bytes);
        const newAttachment = new dntShim.Blob([bytes]);
        const key = this.getKey(formatName, hash);
        this.stagingMap.set(key, newAttachment);
        return Promise.resolve({
            hash,
            size: bytes.byteLength,
            commit: () => {
                this.attachmentMap.set(key, newAttachment);
                this.stagingMap.delete(key);
                return Promise.resolve();
            },
            reject: () => {
                this.stagingMap.delete(key);
                return Promise.resolve();
            },
        });
    }
    erase(formatName, attachmentHash) {
        if (this.closed)
            throw new errors_js_1.ReplicaIsClosedError();
        const key = this.getKey(formatName, attachmentHash);
        if (this.attachmentMap.has(key)) {
            this.attachmentMap.delete(key);
            return Promise.resolve(true);
        }
        return Promise.resolve(new errors_js_1.ValidationError("No attachment with that signature found."));
    }
    wipe() {
        if (this.closed)
            throw new errors_js_1.ReplicaIsClosedError();
        this.attachmentMap.clear();
        return Promise.resolve();
    }
    async filter(hashes) {
        if (this.closed)
            throw new errors_js_1.ReplicaIsClosedError();
        const erasedAttachments = [];
        for (const key of this.attachmentMap.keys()) {
            const [format, hash] = key.split("___");
            const hashesToKeep = hashes[format];
            if (hashesToKeep && !hashesToKeep.has(hash)) {
                const result = await this.erase(format, hash);
                if (result) {
                    erasedAttachments.push({ format, hash });
                }
            }
        }
        return erasedAttachments;
    }
    clearStaging() {
        if (this.closed)
            throw new errors_js_1.ReplicaIsClosedError();
        this.stagingMap.clear();
        return Promise.resolve();
    }
    isClosed() {
        return this.closed;
    }
    async close(erase) {
        if (this.closed)
            throw new errors_js_1.ReplicaIsClosedError();
        if (erase) {
            await this.wipe();
        }
        this.closed = true;
        return;
    }
}
exports.AttachmentDriverMemory = AttachmentDriverMemory;
