"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DocDriverSqlite = void 0;
const errors_js_1 = require("../../util/errors.js");
const sqlite_shared_js_1 = require("./sqlite.shared.js");
const better_sqlite3_1 = __importDefault(require("better-sqlite3"));
const fs = __importStar(require("fs"));
//--------------------------------------------------
const log_js_1 = require("../../util/log.js");
const query_js_1 = require("../../query/query.js");
const compare_js_1 = require("../compare.js");
const addresses_js_1 = require("../../core-validators/addresses.js");
const logger = new log_js_1.Logger("storage driver sqlite node", "yellow");
/** A strorage driver which persists to SQLite. Works in Deno and browsers. */
class DocDriverSqlite {
    //--------------------------------------------------
    // LIFECYCLE
    close(erase) {
        logger.debug("close");
        if (this._isClosed) {
            throw new errors_js_1.ReplicaIsClosedError();
        }
        if (this._db) {
            this._db.close();
        }
        // delete the sqlite file
        if (erase === true && this._filename !== ":memory:") {
            logger.log(`...close: and erase`);
            try {
                fs.rmSync(this._filename);
            }
            catch (err) {
                logger.error("Failed to delete Sqlite file.");
                logger.error(err);
            }
        }
        this._isClosed = true;
        logger.debug("...close is done.");
        return Promise.resolve();
    }
    isClosed() {
        return this._isClosed;
    }
    constructor(opts) {
        Object.defineProperty(this, "share", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "_filename", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "_isClosed", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: false
        });
        Object.defineProperty(this, "_db", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: null
        });
        Object.defineProperty(this, "_maxLocalIndex", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this._filename = opts.filename;
        this.share = "NOT_INITIALIZED";
        // check if file exists
        // check if file exists
        if (opts.mode === "create") {
            if (opts.filename !== ":memory:" && fs.existsSync(opts.filename)) {
                this.close(false);
                throw new errors_js_1.EarthstarError(`Tried to create an sqlite file but it already exists: ${opts.filename}`);
            }
        }
        else if (opts.mode === "open") {
            // this should also fail if you try to open :memory:
            if (!fs.existsSync(opts.filename)) {
                this.close(false);
                throw new errors_js_1.EarthstarError(`Tried to open an sqlite file but it doesn't exist: ${opts.filename}`);
            }
        }
        else if (opts.mode === "create-or-open") {
            // file can exist or not.
        }
        else {
            // unknown mode
            this.close(false);
            throw new errors_js_1.EarthstarError(`sqlite unrecognized opts.mode: ${opts.mode}`);
        }
        const addressIsValidResult = opts.share
            ? (0, addresses_js_1.checkShareIsValid)(opts.share)
            : true;
        if ((0, errors_js_1.isErr)(addressIsValidResult)) {
            throw addressIsValidResult;
        }
        this._db = (0, better_sqlite3_1.default)(this._filename);
        this.ensureTables();
        const maxLocalIndexFromDb = this._db.prepare(sqlite_shared_js_1.MAX_LOCAL_INDEX_QUERY).get()["MAX(localIndex)"];
        // We have to do this because the maxLocalIndexDb could be 0, which is falsy.
        this._maxLocalIndex = maxLocalIndexFromDb !== null
            ? maxLocalIndexFromDb
            : -1;
        // check share
        if (opts.mode === "create") {
            // share is provided; set it into the file which we know didn't exist until just now
            this.share = opts.share;
            this.setConfig("share", this.share);
        }
        else if (opts.mode === "open") {
            // load existing share from file, which we know already existed...
            const existingShare = this._getConfigSync("share");
            if (existingShare === undefined) {
                this.close(false);
                throw new errors_js_1.EarthstarError(`can't open sqlite file with opts.mode="open" because the file doesn't have a share saved in its config table. ${opts.filename}`);
            }
            // if it was also provided in opts, assert that it matches the file
            if (opts.share !== null &&
                opts.share !== this._getConfigSync("share")) {
                this.close(false);
                throw new errors_js_1.EarthstarError(`sqlite with opts.mode="open" wanted share ${opts.share} but found ${existingShare} in the file ${opts.filename}`);
            }
            this.share = existingShare;
        }
        else if (opts.mode === "create-or-open") {
            // share must be provided
            if (opts.share === null) {
                this.close(false);
                throw new errors_js_1.EarthstarError('sqlite with opts.mode="create-or-open" must have a share provided, not null');
            }
            this.share = opts.share;
            // existing share can be undefined (file may not have existed yet)
            const existingShare = this._getConfigSync("share");
            // if there is an existing share, it has to match the one given in opts
            if (existingShare !== undefined &&
                opts.share !== existingShare) {
                this.close(false);
                throw new errors_js_1.EarthstarError(`sqlite file had existing share ${existingShare} but opts wanted it to be ${opts.share} in file ${opts.filename}`);
            }
            // set share if it's not set yet
            if (existingShare === undefined) {
                this.setConfig("share", opts.share);
            }
            this.share = opts.share;
        }
    }
    //--------------------------------------------------
    // CONFIG
    setConfig(key, content) {
        logger.debug(`setConfig(${JSON.stringify(key)} = ${JSON.stringify(content)})`);
        if (this._isClosed) {
            throw new errors_js_1.ReplicaIsClosedError();
        }
        this._db.prepare(sqlite_shared_js_1.UPSERT_CONFIG_QUERY).run({ key: key, content: content });
        return Promise.resolve();
    }
    _getConfigSync(key) {
        const row = this._db.prepare(sqlite_shared_js_1.SELECT_CONFIG_CONTENT_QUERY).get({ key: key });
        const result = row === undefined ? undefined : row.content;
        logger.debug(`getConfig(${JSON.stringify(key)}) = ${JSON.stringify(result)}`);
        return result;
    }
    _listConfigKeysSync() {
        const rows = this._db.prepare(sqlite_shared_js_1.SELECT_KEY_CONFIG_QUERY).all();
        return (0, compare_js_1.sortedInPlace)(rows.map((row) => row.key));
    }
    getConfig(key) {
        if (this._isClosed) {
            throw new errors_js_1.ReplicaIsClosedError();
        }
        return Promise.resolve(this._getConfigSync(key));
    }
    listConfigKeys() {
        if (this._isClosed) {
            throw new errors_js_1.ReplicaIsClosedError();
        }
        return Promise.resolve(this._listConfigKeysSync());
    }
    deleteConfig(key) {
        logger.debug(`deleteConfig(${JSON.stringify(key)})`);
        if (this._isClosed) {
            throw new errors_js_1.ReplicaIsClosedError();
        }
        const result = this._db.prepare(sqlite_shared_js_1.DELETE_CONFIG_QUERY).run({ key: key });
        return Promise.resolve(result.changes > 0);
    }
    //--------------------------------------------------
    // GET
    getMaxLocalIndex() {
        if (this._isClosed) {
            throw new errors_js_1.ReplicaIsClosedError();
        }
        return Promise.resolve(this._maxLocalIndex);
    }
    queryDocs(queryToClean) {
        return Promise.resolve(this.queryDocsSync(queryToClean));
    }
    queryDocsSync(queryToClean) {
        // Query the documents
        logger.debug("queryDocs", queryToClean);
        if (this._isClosed) {
            throw new errors_js_1.ReplicaIsClosedError();
        }
        // clean up the query and exit early if possible.
        const { query, willMatch } = (0, query_js_1.cleanUpQuery)(queryToClean);
        logger.debug(`    cleanUpQuery.  willMatch = ${willMatch}`);
        if (willMatch === "nothing") {
            return [];
        }
        const now = Date.now() * 1000;
        const { sql, params } = (0, sqlite_shared_js_1.makeDocQuerySql)(query, now, "documents");
        logger.debug("  sql:", sql);
        logger.debug("  params:", params);
        const docRows = this._db.prepare(sql).all(params);
        const docs = [];
        for (const row of docRows) {
            const doc = JSON.parse(row.doc);
            docs.push({ ...doc, _localIndex: row.localIndex });
        }
        return docs;
    }
    //--------------------------------------------------
    // SET
    upsert(doc) {
        return Promise.resolve(this.upsertSync(doc));
    }
    upsertSync(doc) {
        // Insert new doc, replacing old doc if there is one
        logger.debug(`upsertDocument(doc.path: ${JSON.stringify(doc.path)})`);
        if (this._isClosed) {
            throw new errors_js_1.ReplicaIsClosedError();
        }
        Object.freeze(doc);
        const row = {
            doc: JSON.stringify(doc),
            localIndex: this._maxLocalIndex + 1,
            pathAuthor: `${doc.path} ${doc.author}`,
        };
        this._maxLocalIndex += 1;
        this._db.prepare(sqlite_shared_js_1.UPSERT_DOC_QUERY).run(row);
        return { ...doc, _localIndex: row.localIndex };
    }
    eraseExpiredDocs() {
        if (this._isClosed) {
            throw new errors_js_1.ReplicaIsClosedError();
        }
        const now = Date.now() * 1000;
        const docsToWipe = this._db.prepare(sqlite_shared_js_1.SELECT_EXPIRED_DOC_QUERY).all({ now });
        this._db.prepare(sqlite_shared_js_1.DELETE_EXPIRED_DOC_QUERY).run({ now });
        const docs = [];
        for (const row of docsToWipe) {
            docs.push(JSON.parse(row.doc));
        }
        return Promise.resolve(docs);
    }
    //--------------------------------------------------
    // SQL STUFF
    ensureTables() {
        if (this._isClosed) {
            throw new errors_js_1.ReplicaIsClosedError();
        }
        // make sure sqlite is using utf-8
        const encoding = this._db.pragma("encoding", { simple: true });
        if (encoding !== "UTF-8") {
            throw new Error(`sqlite encoding is stubbornly set to ${encoding} instead of UTF-8`);
        }
        this._db.prepare(sqlite_shared_js_1.CREATE_CONFIG_TABLE_QUERY).run();
        // check and set schemaVersion
        let schemaVersion = this._getConfigSync("schemaVersion");
        logger.log(`constructor    schemaVersion: ${schemaVersion}`);
        let docsToMigrate = [];
        if (schemaVersion === undefined) {
            schemaVersion = "2";
            this.setConfig("schemaVersion", schemaVersion);
        }
        else if (schemaVersion !== "2") {
            // MIGRATE.
            docsToMigrate = this.queryDocsSync({
                historyMode: "all",
                orderBy: "localIndex ASC",
            });
            this._db.prepare(`DROP TABLE docs;`).run();
        }
        this._db.prepare(sqlite_shared_js_1.CREATE_DOCS_TABLE_QUERY).run();
        const indexStatements = sqlite_shared_js_1.CREATE_INDEXES_QUERY.split("\n");
        // bettersqlite3 needs a single statement per call to .prepare
        for (const statement of indexStatements) {
            const trimmed = statement.trim();
            if (trimmed.length > 0) {
                this._db.prepare(statement).run();
            }
        }
        for (const doc of docsToMigrate) {
            this.upsertSync(doc);
        }
    }
}
exports.DocDriverSqlite = DocDriverSqlite;
