"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExtensionServerSettings = void 0;
const dntShim = __importStar(require("../../../_dnt.shims.js"));
const addresses_js_1 = require("../../core-validators/addresses.js");
const crypto_js_1 = require("../../crypto/crypto.js");
const query_helpers_js_1 = require("../../query/query-helpers.js");
const errors_js_1 = require("../../util/errors.js");
const HOSTED_SHARE_TEMPLATE = `/server-settings/1.*/shares/{shareHash}/{hostType}`;
/** A server extension which reads settings from a specified share, e.g. which shares to host on the server. Settings are modified by other peers syncing with this server. */
class ExtensionServerSettings {
    constructor(opts) {
        Object.defineProperty(this, "configReplica", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "onCreateReplica", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "peer", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: null
        });
        this.configReplica = opts.onCreateReplica(opts.settingsShare);
        this.onCreateReplica = opts.onCreateReplica;
    }
    register(peer) {
        this.peer = peer;
        peer.addReplica(this.configReplica);
        const onCreateReplica = this.onCreateReplica;
        const removeShareWithHash = this.removeShareWithHash.bind(this);
        const { glob } = (0, query_helpers_js_1.parseTemplate)(HOSTED_SHARE_TEMPLATE);
        const { query, regex } = (0, query_helpers_js_1.globToQueryAndRegex)(glob);
        const configShareAddress = this.configReplica.share;
        this.configReplica.getQueryStream(query, "everything").pipeTo(new dntShim.WritableStream({
            async write(event) {
                if (event.kind === "existing" || event.kind === "success") {
                    if (regex != null &&
                        new RegExp(regex).test(event.doc.path)) {
                        const pathVariables = (0, query_helpers_js_1.extractTemplateVariablesFromPath)(HOSTED_SHARE_TEMPLATE, event.doc.path);
                        if (!pathVariables) {
                            return;
                        }
                        const shareHash = pathVariables["shareHash"];
                        const configHash = await crypto_js_1.Crypto.sha256base32(configShareAddress);
                        if (shareHash === configHash) {
                            return;
                        }
                        if (event.doc.text.length > 0 &&
                            (0, errors_js_1.notErr)((0, addresses_js_1.parseShareAddress)(event.doc.text)) &&
                            !peer.hasShare(event.doc.text)) {
                            // Add the share
                            const replica = onCreateReplica(event.doc.text);
                            peer.addReplica(replica);
                            console.log("Server settings:", `now hosting ${replica.share}`);
                        }
                        else if (event.doc.text.length === 0) {
                            // Remove the share
                            await removeShareWithHash(shareHash);
                        }
                    }
                }
                if (event.kind === "expire") {
                    if (regex != null &&
                        new RegExp(regex).test(event.doc.path)) {
                        // Remove the share
                        const pathVariables = (0, query_helpers_js_1.extractTemplateVariablesFromPath)(HOSTED_SHARE_TEMPLATE, event.doc.path);
                        if (!pathVariables) {
                            return;
                        }
                        const shareHash = pathVariables["shareHash"];
                        await removeShareWithHash(shareHash);
                    }
                }
            },
        }));
        return Promise.resolve();
    }
    handler(_req) {
        return Promise.resolve(null);
    }
    async removeShareWithHash(hash) {
        const maybeReplicaForHash = await this.findReplicaForHash(hash);
        if (!maybeReplicaForHash) {
            return;
        }
        // Now we find all docs with this hash.
        // If any of them have text defined on them, we bail.
        const ephemeralDoc = await this.configReplica.queryDocs({
            filter: {
                pathStartsWith: "/server-settings/1.",
                pathEndsWith: `/shares/${hash}/host!`,
            },
        });
        const nonEphemeralDoc = await this.configReplica.queryDocs({
            filter: {
                pathStartsWith: "/server-settings/1.",
                pathEndsWith: `/shares/${hash}/host`,
            },
        });
        if (ephemeralDoc[0] && ephemeralDoc[0].text.length > 0) {
            return;
        }
        if (nonEphemeralDoc[0] && nonEphemeralDoc[0].text.length > 0) {
            return;
        }
        // Now we know all docs for this share are empty. Remove.
        await maybeReplicaForHash.close(true);
        this.peer?.removeReplica(maybeReplicaForHash);
        console.log("Server settings:", `stopped hosting ${maybeReplicaForHash.share}`);
    }
    async findReplicaForHash(hash) {
        for (const replica of this.peer?.replicas() || []) {
            const rHash = await crypto_js_1.Crypto.sha256base32(replica.share);
            if (rHash === hash) {
                return replica;
            }
        }
    }
}
exports.ExtensionServerSettings = ExtensionServerSettings;
