"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExtensionSyncWeb = void 0;
const dntShim = __importStar(require("../../../_dnt.shims.js"));
const deferred_js_1 = require("../../../deps/deno.land/std@0.138.0/async/deferred.js");
const partner_web_client_js_1 = require("../../syncer/partner_web_client.js");
const misc_js_1 = require("../../util/misc.js");
const ws_1 = require("ws");
/** An extension which enables synchronisation over the web via HTTP. */
class ExtensionSyncWeb {
    constructor(opts) {
        Object.defineProperty(this, "path", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: ""
        });
        Object.defineProperty(this, "syncers", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Map()
        });
        Object.defineProperty(this, "peer", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: (0, deferred_js_1.deferred)()
        });
        Object.defineProperty(this, "formats", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "wss", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "server", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        if (opts?.path) {
            this.path = opts.path;
        }
        if (opts?.formats) {
            this.formats = opts.formats;
        }
        this.server = opts.server;
        this.wss = new ws_1.WebSocketServer({ noServer: true });
    }
    register(peer) {
        this.peer.resolve(peer);
        const transferPattern = new dntShim.URLPattern({
            pathname: `${this.path}/:syncerId/:kind/:shareAddress/:formatName/:author/:path*`,
        });
        const initiatePattern = new dntShim.URLPattern({
            pathname: `${this.path}/:mode`,
        });
        this.server.on("upgrade", (req, socket, head) => {
            const reqUrl = `http://0.0.0.0${req.url}`;
            if (transferPattern.test(reqUrl) || initiatePattern.test(reqUrl)) {
                this.wss.handleUpgrade(req, socket, head, (ws) => {
                    // @ts-ignore
                    this.wss.emit("connection", ws, req);
                });
            }
        });
        this.wss.on("connection", async (socket, req) => {
            const reqUrl = `http://0.0.0.0${req.url}`;
            const transferMatch = transferPattern.exec(reqUrl);
            if (transferMatch) {
                const { syncerId, shareAddress, formatName, path, author, kind } = transferMatch.pathname.groups;
                const syncer = this.syncers.get(syncerId);
                if (!syncer) {
                    return;
                }
                await syncer.handleTransferRequest({
                    shareAddress,
                    formatName,
                    path: `/${path}`,
                    author,
                    kind: kind,
                    source: socket,
                });
                return;
            }
            const initiateMatch = initiatePattern.exec(reqUrl);
            if (initiateMatch) {
                const { mode } = initiateMatch.pathname.groups;
                if (mode !== "once" && mode !== "continuous") {
                    return;
                }
                const peer = await this.peer;
                const partner = new partner_web_client_js_1.PartnerWebClient({
                    socket,
                    appetite: mode === "once" ? "once" : "continuous",
                });
                const description = `Client ${(0, misc_js_1.randomId)()}`;
                const newSyncer = peer.addSyncPartner(partner, description, this.formats);
                console.log(`${description}: started sync`);
                newSyncer.isDone().then(() => {
                    console.log(`${description}: completed sync`);
                }).catch((err) => {
                    console.error(`Syncer ${newSyncer.id}: cancelled`, err);
                }).finally(() => {
                    console.log(`${description}: removed`);
                    this.syncers.delete(newSyncer.id);
                });
                this.syncers.set(newSyncer.id, newSyncer);
            }
        });
        return Promise.resolve();
    }
    async handler(req) {
        return Promise.resolve(null);
    }
}
exports.ExtensionSyncWeb = ExtensionSyncWeb;
