"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Server = void 0;
const dntShim = __importStar(require("../../_dnt.shims.js"));
const node_buffer_1 = require("node:buffer");
const server_core_js_1 = require("./server_core.js");
/**
 * An extensible server able to synchronise with other peers.
 *
 * A server's functionality can be extended using extensions of type `IServerExtension`.
 */
class Server {
    /**
     * Create a new server with an array of extensions.
     * @param extensions - The extensions used by the server. Extensions will be registered in the order you provide them in, as one extension may depend on the actions of another. For example, the `ExtensionServeContent` may rely on a replica created by `ExtensionKnownShares`.
     */
    constructor(extensions, opts) {
        Object.defineProperty(this, "core", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "server", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.core = new server_core_js_1.ServerCore(extensions);
        this.server = opts.server;
        this.server.on("request", async (req, res) => {
            let data = undefined;
            if (req.method === "POST") {
                const buffers = [];
                for await (const chunk of req) {
                    buffers.push(chunk);
                }
                data = node_buffer_1.Buffer.concat(buffers).toString();
            }
            const headers = new dntShim.Headers();
            for (const key in req.headers) {
                if (req.headers[key])
                    headers.append(key, req.headers[key]);
            }
            // Need the hostname here so the URL plays nice with Node's URL class.
            const url = `http://0.0.0.0${req.url}`;
            const request = new dntShim.Request(url, {
                method: req.method,
                headers,
                body: data,
            });
            const response = await this.core.handler(request);
            // Headers
            for (const [key, value] of response.headers) {
                res.setHeader(key, value);
            }
            // Status
            res.statusCode = response.status;
            // Body
            // TODO: Handle streaming responses.
            if (response.body) {
                res.end(response.body);
            }
            res.end();
        });
        this.server.listen(opts?.port);
    }
    close() {
        this.server.close();
    }
}
exports.Server = Server;
