"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.websocketReadable = exports.websocketWritable = exports.StreamSplitter = exports.ChannelMultiStream = exports.ChannelTransformer = exports.LockStream = exports.PassThroughTransformer = exports.BlockingBus = exports.CallbackSink = exports.MultiStream = exports.CloneMidStream = exports.CloneStream = exports.CombineStream = void 0;
const dntShim = __importStar(require("../../_dnt.shims.js"));
const deps_js_1 = require("../../deps.js");
const promise_enroller_js_1 = require("../syncer/promise_enroller.js");
class CombineStream {
    constructor() {
        Object.defineProperty(this, "closed", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: false
        });
        Object.defineProperty(this, "transform", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new dntShim.TransformStream({
                transform(chunk, controller) {
                    controller.enqueue(chunk);
                },
            })
        });
        Object.defineProperty(this, "writer", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: this.transform.writable.getWriter()
        });
        Object.defineProperty(this, "readable", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: this.transform.readable
        });
        Object.defineProperty(this, "writables", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: []
        });
    }
    checkClosed() {
        if (this.closed) {
            throw "Closed";
        }
    }
    getWritableStream() {
        this.checkClosed();
        const writer = this.writer;
        const writable = new dntShim.WritableStream({
            async write(chunk) {
                await writer.ready;
                await writer.write(chunk);
            },
        });
        this.writables.push(writable);
        return writable;
    }
    async close() {
        this.checkClosed();
        for (const writableStream of this.writables) {
            await writableStream.abort();
        }
        this.transform.writable.abort();
        this.closed = true;
    }
    isClosed() {
        return this.closed;
    }
}
exports.CombineStream = CombineStream;
class CloneStream {
    constructor() {
        Object.defineProperty(this, "transformStream", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new dntShim.TransformStream(new PassThroughTransformer())
        });
        Object.defineProperty(this, "sourceReadable", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: this.transformStream.readable
        });
        Object.defineProperty(this, "writable", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: this.transformStream.writable
        });
    }
    getReadableStream() {
        const [r1, r2] = this.sourceReadable.tee();
        this.sourceReadable = r1;
        return r2;
    }
}
exports.CloneStream = CloneStream;
// Only gets chunks from the writable end AFTER the readable stream is constructed.
class CloneMidStream {
    checkClosed() {
        if (this.closed) {
            throw "Closed";
        }
    }
    constructor() {
        Object.defineProperty(this, "closed", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: false
        });
        Object.defineProperty(this, "subscribers", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: []
        });
        Object.defineProperty(this, "writables", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: []
        });
        Object.defineProperty(this, "writable", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        const subscribers = this.subscribers;
        this.writable = new dntShim.WritableStream({
            write(chunk) {
                const notifies = subscribers.map(({ writer }) => {
                    return new Promise((resolve) => {
                        writer.ready.then(() => {
                            writer.write(chunk).then(resolve);
                        });
                    });
                });
                Promise.all(notifies);
            },
        });
    }
    getReadableStream() {
        this.checkClosed();
        const transform = new dntShim.TransformStream({
            transform(chunk, controller) {
                controller.enqueue(chunk);
            },
        });
        this.subscribers.push({
            transform,
            writer: transform.writable.getWriter(),
        });
        return transform.readable;
    }
    async close() {
        this.checkClosed();
        for (const writableStream of this.writables) {
            await writableStream.abort();
        }
        for (const { transform } of this.subscribers) {
            await transform.writable.abort();
            await transform.readable.cancel();
        }
        this.closed = true;
    }
    isClosed() {
        return this.closed;
    }
}
exports.CloneMidStream = CloneMidStream;
class MultiStream {
    checkClosed() {
        if (this.closed) {
            throw "Closed";
        }
    }
    constructor(joinMidStream) {
        Object.defineProperty(this, "closed", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: false
        });
        Object.defineProperty(this, "combineStream", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new CombineStream()
        });
        Object.defineProperty(this, "cloneStream", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        if (joinMidStream) {
            this.cloneStream = new CloneMidStream();
        }
        else {
            this.cloneStream = new CloneStream();
        }
        this.combineStream.readable.pipeTo(this.cloneStream.writable);
    }
    getWritableStream() {
        return this.combineStream.getWritableStream();
    }
    getReadableStream() {
        return this.cloneStream.getReadableStream();
    }
    close() {
        this.checkClosed();
        this.combineStream.close();
        //this.cloneStream.close();
        this.closed = true;
    }
    isClosed() {
        return this.closed;
    }
}
exports.MultiStream = MultiStream;
class CallbackSink {
    constructor() {
        Object.defineProperty(this, "callbacks", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Set()
        });
    }
    onWrite(callback) {
        this.callbacks.add(callback);
        return () => {
            this.callbacks.delete(callback);
        };
    }
    async write(chunk) {
        for (const callback of this.callbacks) {
            await callback(chunk);
        }
    }
    close() {
        this.callbacks.clear();
    }
}
exports.CallbackSink = CallbackSink;
// This bus runs all its callbacks blockingly, so that no two subscribed callbacks are ever called at the same time.
class BlockingBus {
    constructor() {
        Object.defineProperty(this, "callbacks", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Set()
        });
        Object.defineProperty(this, "lock", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new LockStream()
        });
    }
    on(cb) {
        this.callbacks.add(cb);
        return () => {
            this.callbacks.delete(cb);
        };
    }
    async send(item) {
        for (const callback of this.callbacks) {
            await this.lock.run(() => callback(item));
        }
    }
}
exports.BlockingBus = BlockingBus;
class PassThroughTransformer {
    transform(chunk, controller) {
        controller.enqueue(chunk);
    }
}
exports.PassThroughTransformer = PassThroughTransformer;
class LockStream {
    constructor() {
        Object.defineProperty(this, "doneEnroller", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new promise_enroller_js_1.PromiseEnroller()
        });
        Object.defineProperty(this, "writable", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new dntShim.WritableStream({
                async write(cb) {
                    await cb();
                },
            })
        });
        Object.defineProperty(this, "writer", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: this.writable.getWriter()
        });
        Object.defineProperty(this, "closed", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: false
        });
    }
    checkClosed() {
        if (this.closed) {
            throw "Closed";
        }
    }
    async run(cb) {
        this.checkClosed();
        await this.writer.ready;
        const written = this.writer.write(cb);
        this.doneEnroller.enrol(written);
        return written;
    }
    async close() {
        this.checkClosed();
        this.doneEnroller.seal();
        // Wait for it to be closed.
        await this.doneEnroller.isDone();
        this.writer.releaseLock();
        await this.writable.abort();
        this.closed = true;
    }
    isClosed() {
        return this.closed;
    }
}
exports.LockStream = LockStream;
class ChannelTransformer {
    constructor(channel, key) {
        Object.defineProperty(this, "channel", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "channelKey", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.channel = channel;
        this.channelKey = key;
    }
    transform(chunk, controller) {
        if (this.channel === "*" || this.channel === chunk[this.channelKey]) {
            controller.enqueue(chunk);
            return;
        }
    }
}
exports.ChannelTransformer = ChannelTransformer;
class ChannelMultiStream {
    constructor(key, joinMidStream) {
        Object.defineProperty(this, "multistream", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "channelKey", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.multistream = new MultiStream(joinMidStream);
        this.channelKey = key;
    }
    getWritableStream() {
        return this.multistream.getWritableStream();
    }
    getReadableStream(channel) {
        const channelTransform = new dntShim.TransformStream(new ChannelTransformer(channel, this.channelKey));
        return this.multistream.getReadableStream().pipeThrough(channelTransform);
    }
}
exports.ChannelMultiStream = ChannelMultiStream;
class StreamSplitter {
    constructor(getKey) {
        Object.defineProperty(this, "transforms", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Map()
        });
        Object.defineProperty(this, "incomingCloner", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new CloneStream()
        });
        Object.defineProperty(this, "writable", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: this.incomingCloner.writable
        });
        Object.defineProperty(this, "getKey", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.getKey = getKey;
    }
    getReadable(key) {
        const { transforms, incomingCloner, getKey } = this;
        const transform = transforms.get(key);
        if (!transform) {
            const incomingClone = incomingCloner.getReadableStream();
            const newTransform = new dntShim.TransformStream(new PassThroughTransformer());
            const filterTransform = new dntShim.TransformStream({
                transform(chunk, controller) {
                    if (getKey(chunk) === key) {
                        controller.enqueue(chunk);
                    }
                },
            });
            incomingClone.pipeThrough(filterTransform).pipeTo(newTransform.writable)
                .catch((err) => {
                newTransform.writable.abort(err);
            });
            transforms.set(key, newTransform);
            return newTransform.readable;
        }
        return transform.readable;
    }
}
exports.StreamSplitter = StreamSplitter;
function websocketWritable(socketOrUrl, prepareToSend) {
    const socketIsOpen = (0, deps_js_1.deferred)();
    const initialSocket = (0, deps_js_1.deferred)();
    const setUpSocket = () => {
        if (initialSocket.state === "pending") {
            let socket;
            if (socketOrUrl instanceof dntShim.WebSocket) {
                socket = socketOrUrl;
            }
            else {
                socket = new dntShim.WebSocket(socketOrUrl);
            }
            initialSocket.resolve(socket);
            // set socket binary type
            socket.binaryType = "arraybuffer";
            // set socket.onopen
            if (socket.readyState === socket.OPEN) {
                socketIsOpen.resolve();
            }
            socket.onopen = () => {
                socketIsOpen.resolve();
            };
        }
    };
    return new dntShim.WritableStream({
        async write(chunk, controller) {
            setUpSocket();
            const socket = await initialSocket;
            await socketIsOpen;
            // try to send
            try {
                const toSend = prepareToSend(chunk);
                socket.send(toSend);
            }
            catch (err) {
                controller.error(err);
            }
            socket.onclose = () => {
                controller.error("Socket closed before we were done");
            };
        },
        async close() {
            setUpSocket();
            const socket = await initialSocket;
            await socketIsOpen;
            socket.close();
        },
        async abort() {
            setUpSocket();
            const socket = await initialSocket;
            await socketIsOpen;
            socket.close();
        },
    });
}
exports.websocketWritable = websocketWritable;
function websocketReadable(socketOrUrl, prepareForQueue) {
    const initialSocket = (0, deps_js_1.deferred)();
    let erroredOutAlready = false;
    const setupSocket = () => {
        if (initialSocket.state === "pending") {
            if (socketOrUrl instanceof dntShim.WebSocket) {
                initialSocket.resolve(socketOrUrl);
            }
            else {
                try {
                    const socket = new dntShim.WebSocket(socketOrUrl);
                    initialSocket.resolve(socket);
                }
                catch (err) {
                    initialSocket.reject(err);
                }
            }
        }
    };
    return new dntShim.ReadableStream({
        async start(controller) {
            setupSocket();
            const socket = await initialSocket;
            // set socket binary type
            socket.binaryType = "arraybuffer";
            socket.onmessage = (event) => {
                const toQueue = prepareForQueue(event);
                try {
                    controller.enqueue(toQueue);
                }
                catch {
                    // The stream was probably closed by something else.
                }
            };
            socket.onclose = () => {
                if (erroredOutAlready) {
                    return;
                }
                try {
                    controller.close();
                }
                catch {
                    // The stream was probably closed by something else already.
                }
            };
            socket.onerror = (err) => {
                erroredOutAlready = true;
                controller.error(err);
            };
        },
        async cancel() {
            setupSocket();
            const socket = await initialSocket;
            socket.close();
        },
    });
}
exports.websocketReadable = websocketReadable;
