"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AttachmentTransfer = void 0;
const dntShim = __importStar(require("../../_dnt.shims.js"));
const deps_js_1 = require("../../deps.js");
const stream_utils_js_1 = require("../streams/stream_utils.js");
const errors_js_1 = require("../util/errors.js");
const bumping_timeout_js_1 = require("./bumping_timeout.js");
const constants_js_1 = require("./constants.js");
const multi_deferred_js_1 = require("./multi_deferred.js");
class AttachmentTransfer {
    constructor({ stream, replica, doc, format, requester, counterpartId }) {
        Object.defineProperty(this, "kind", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "status", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: "ready"
        });
        Object.defineProperty(this, "share", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "loaded", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
        Object.defineProperty(this, "expectedSize", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "sourceDoc", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "statusBus", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new stream_utils_js_1.BlockingBus()
        });
        Object.defineProperty(this, "transferOp", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: (0, deps_js_1.deferred)()
        });
        Object.defineProperty(this, "abortCb", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "multiDeferred", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new multi_deferred_js_1.MultiDeferred()
        });
        Object.defineProperty(this, "hash", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "requester", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.sourceDoc = doc;
        this.share = replica.share;
        this.requester = requester;
        const attachmentInfo = format.getAttachmentInfo(doc);
        if ((0, errors_js_1.isErr)(attachmentInfo)) {
            throw new errors_js_1.ValidationError("AttachmentTransfer was given a doc which has no attachment!");
        }
        this.hash = attachmentInfo.hash;
        this.expectedSize = attachmentInfo.size;
        const updateLoaded = this.updateLoaded.bind(this);
        if (stream instanceof dntShim.ReadableStream) {
            // Incoming
            // pipe through our bytes counter
            this.kind = "download";
            let reader;
            this.abortCb = () => {
                if (reader) {
                    reader.cancel();
                }
                else {
                    stream.cancel();
                }
            };
            let bumpingTimeout = null;
            const counterStream = new dntShim.ReadableStream({
                async start(controller) {
                    const newReader = stream.getReader();
                    // @ts-ignore Node's ReadableStream types does not like this for some reason.
                    reader = newReader;
                    bumpingTimeout = new bumping_timeout_js_1.BumpingTimeout(() => {
                        controller.error("Attachment download timed out.");
                    }, constants_js_1.TIMEOUT_MS);
                    while (true) {
                        const { done, value } = await newReader.read();
                        // Clear timeout here.
                        bumpingTimeout.bump();
                        if (done) {
                            break;
                        }
                        updateLoaded(value.byteLength);
                        controller.enqueue(value);
                    }
                    bumpingTimeout.close();
                    controller.close();
                },
            });
            this.transferOp.resolve(() => {
                const promise = (0, deps_js_1.deferred)();
                replica.ingestAttachment(format, doc, counterStream, counterpartId)
                    .then((result) => {
                    if ((0, errors_js_1.isErr)(result) && this.loaded === 0) {
                        // The other peer didn't have this attachment.
                        this.changeStatus("missing_attachment");
                        return;
                    }
                    if ((0, errors_js_1.isErr)(result)) {
                        console.warn(`Couldn't ingest the attachment for ${doc.path} by ${doc.author}: ${result.message}`);
                        promise.reject(result);
                        return;
                    }
                    promise.resolve();
                }).catch((err) => {
                    promise.reject(err);
                });
                return promise;
            });
        }
        else {
            this.kind = "upload";
            this.abortCb = () => {
                stream.abort().catch(() => {
                    // Node doesn't let you abort a locked writablestream (wrong)
                    // https://github.com/nodejs/node/issues/41159
                });
            };
            replica.getAttachment(doc, format).then(async (attachmentRes) => {
                if (!attachmentRes) {
                    await this.changeStatus("missing_attachment");
                    await stream.abort();
                    return;
                }
                if ((0, errors_js_1.isErr)(attachmentRes)) {
                    await this.changeStatus("failed");
                    await stream.abort();
                    return;
                }
                const counterTransform = new dntShim.TransformStream({
                    transform(chunk, controller) {
                        updateLoaded(chunk.byteLength);
                        controller.enqueue(chunk);
                    },
                });
                this.transferOp.resolve(() => attachmentRes.stream().then((readable) => {
                    return readable.pipeThrough(counterTransform).pipeTo(stream);
                }));
            });
        }
    }
    async start() {
        if (this.status !== "ready") {
            // maybe throw here.
            return;
        }
        const transferOp = await this.transferOp;
        await this.changeStatus("in_progress");
        transferOp().then(async () => {
            await this.changeStatus("complete");
        }).catch(async () => {
            await this.changeStatus("failed");
        });
        return;
    }
    updateLoaded(toAdd) {
        this.loaded += toAdd;
        this.statusBus.send({
            status: this.status,
            bytesLoaded: this.loaded,
            totalBytes: this.expectedSize,
        });
    }
    async changeStatus(status) {
        if (this.status === "complete" || this.status === "failed" ||
            this.status === "missing_attachment") {
            return;
        }
        this.status = status;
        await this.statusBus.send({
            status: status,
            bytesLoaded: this.loaded,
            totalBytes: this.expectedSize,
        });
        if (status === "complete") {
            this.multiDeferred.resolve();
        }
        if (status === "failed") {
            this.multiDeferred.reject("Attachment transfer failed");
        }
        if (status === "missing_attachment") {
            this.multiDeferred.reject("The other peer does not have this attachment");
        }
    }
    get doc() {
        return this.sourceDoc;
    }
    onProgress(callback) {
        const unsub = this.statusBus.on(callback);
        return unsub;
    }
    abort() {
        this.abortCb();
    }
    isDone() {
        return this.multiDeferred.getPromise();
    }
}
exports.AttachmentTransfer = AttachmentTransfer;
