"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MultiDeferred = void 0;
const deps_js_1 = require("../../deps.js");
class MultiDeferred {
    constructor() {
        Object.defineProperty(this, "deferreds", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Set()
        });
        Object.defineProperty(this, "state", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: "pending"
        });
    }
    resolve(value) {
        if (this.state !== "pending") {
            return;
        }
        this.state = "fulfilled";
        for (const deferred of this.deferreds) {
            deferred.resolve(value);
        }
    }
    reject(reason) {
        if (this.state !== "pending") {
            return;
        }
        this.state = "rejected";
        for (const deferred of this.deferreds) {
            deferred.reject(reason);
        }
    }
    getPromise() {
        const promise = (0, deps_js_1.deferred)();
        if (this.state === "fulfilled") {
            promise.resolve();
        }
        else if (this.state === "rejected") {
            promise.reject();
        }
        else {
            this.deferreds.add(promise);
        }
        return promise;
    }
}
exports.MultiDeferred = MultiDeferred;
