"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PartnerWebServer = void 0;
const dntShim = __importStar(require("../../_dnt.shims.js"));
const deps_js_1 = require("../../deps.js");
const stream_utils_js_1 = require("../streams/stream_utils.js");
const errors_js_1 = require("../util/errors.js");
/** A syncing partner to be used with servers reachable via the internet.
 * Works everywhere.
 */
class PartnerWebServer {
    constructor(opts) {
        Object.defineProperty(this, "syncAppetite", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "concurrentTransfers", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 16
        });
        Object.defineProperty(this, "payloadThreshold", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 8
        });
        Object.defineProperty(this, "rangeDivision", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 8
        });
        Object.defineProperty(this, "isSecure", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "wsUrl", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "socket", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "incomingQueue", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new deps_js_1.AsyncQueue()
        });
        Object.defineProperty(this, "socketIsReady", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: (0, deps_js_1.deferred)()
        });
        this.syncAppetite = opts.appetite;
        // Check if it's a URL of some kind.
        const url = new URL(opts.url);
        // Check if it's a web syncer
        const hostAndPath = `${url.host}${url.pathname === "/" ? "" : url.pathname}`;
        this.isSecure = url.protocol === "https:" ||
            url.protocol === "wss:";
        this.wsUrl = `${this.isSecure ? "wss://" : "ws://"}${hostAndPath}/'`;
        const urlWithMode = new URL(opts.appetite, this.wsUrl);
        this.socket = new dntShim.WebSocket(urlWithMode.toString());
        this.socket.onopen = () => {
            this.socketIsReady.resolve();
        };
        this.socket.binaryType = "arraybuffer";
        this.socket.onmessage = (event) => {
            // Casting to string due to Node's incorrect websocket types
            this.incomingQueue.push(JSON.parse(event.data));
        };
        this.socket.onclose = () => {
            this.incomingQueue.close();
        };
        this.socket.onerror = (event) => {
            if ("error" in event) {
                this.incomingQueue.close({
                    withError: event.error,
                });
                return;
            }
            this.incomingQueue.close({
                withError: new errors_js_1.EarthstarError("Websocket error."),
            });
        };
    }
    async sendEvent(event) {
        await this.socketIsReady;
        if (this.socket.readyState === this.socket.CLOSED ||
            this.socket.readyState === this.socket.CLOSING) {
            return;
        }
        return this.socket.send(JSON.stringify(event));
    }
    getEvents() {
        return this.incomingQueue;
    }
    closeConnection() {
        this.socket.close();
        return Promise.resolve();
    }
    getDownload(opts) {
        // create a new url with the share, path, and syncer ID embedded
        const url = new URL(`${opts.syncerId}/download/${opts.shareAddress}/${opts.doc.format}/${opts.doc.author}${opts.doc.path}`, this.wsUrl);
        const readable = (0, stream_utils_js_1.websocketReadable)(url.toString(), (event) => {
            if (event.data instanceof ArrayBuffer) {
                const bytes = new Uint8Array(event.data);
                return bytes;
            }
            return null;
        });
        return Promise.resolve(readable);
    }
    handleUploadRequest(opts) {
        const url = new URL(`${opts.syncerId}/upload/${opts.shareAddress}/${opts.doc.format}/${opts.doc.author}${opts.doc.path}`, this.wsUrl);
        const writable = (0, stream_utils_js_1.websocketWritable)(url.toString(), (outgoing) => outgoing);
        return Promise.resolve(writable);
    }
    handleTransferRequest(_source, _kind) {
        // We don't expect any external requests.
        return Promise.resolve(new errors_js_1.NotSupportedError("SyncDriverWebClient does not support external transfer requests."));
    }
}
exports.PartnerWebServer = PartnerWebServer;
