"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PlumTree = void 0;
const dntShim = __importStar(require("../../_dnt.shims.js"));
/** A push-lazy-push-multicast tree, or 'PlumTree'. Organises a network of interconnected peers into a spanning tree where messaging resiliency, latency, and bandwidth are optimised.
 *
 * When Earthstar sync agents finish their initial reconciliation phase they are switched to a mode where they are managed by a plumtree.
 */
class PlumTree {
    constructor() {
        Object.defineProperty(this, "messagingModes", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Map()
        });
        /** A list of previously received message IDs, used to check incoming messages for duplicates.
         */
        Object.defineProperty(this, "eagerMessageThumbnails", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Set()
        });
        /** A map of DocThumbnails to timers. */
        Object.defineProperty(this, "haveTimeouts", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Map()
        });
    }
    getMode(id) {
        const maybeMode = this.messagingModes.get(id);
        if (maybeMode) {
            return maybeMode;
        }
        const initialMode = this.messagingModes.size === 0 ? "EAGER" : "LAZY";
        this.messagingModes.set(id, initialMode);
        return "EAGER";
    }
    /** Triggered when the other peer sends a DOC message. Returns a boolean indicating whether to send a PRUNE event to the peer we got this message from. */
    onEagerMessage(id, event) {
        // Check the list of have timers to see if we're waiting for this event.
        // If it's there, clear the timer.
        const maybeTimeout = this.haveTimeouts.get(event.thumbnail);
        if (maybeTimeout) {
            clearTimeout(maybeTimeout);
            // TODO: stop here?
            return false;
        }
        // Check the list of previously received eager messages for this ID.
        if (this.eagerMessageThumbnails.has(event.thumbnail)) {
            // If already present, switch to lazy messaging this peer.
            this.messagingModes.set(id, "LAZY");
            // ask syncagent to tell partner to make become lazy (PRUNE).
            return true;
        }
        else {
            // If not present, add to the list.
            this.eagerMessageThumbnails.add(event.thumbnail);
            return false;
        }
    }
    /** Triggered when the other peer sends a HAVE message */
    onLazyMessage(event, dispatchGraft) {
        // Set a timer for the this message to arrive.
        // (if a timer already exists, do nothing)
        if (!this.haveTimeouts.has(event.thumbnail)) {
            const timeout = dntShim.setTimeout(() => {
                // When the timer expires, get the syncagent to send a WANT for this thumbnail.
                dispatchGraft(event.thumbnail);
            }, 5);
            // TODO: What should the timeout be?
            this.haveTimeouts.set(event.thumbnail, timeout);
        }
    }
    /** Triggered when the other peer sends a graft, i.e. WANT message. */
    onGraftMessage(id) {
        // Move the peer who requested this to our eager peers.
        this.messagingModes.set(id, "EAGER");
    }
    /** Triggered when the other peer sends a PRUNE message. */
    onPrune(id) {
        this.messagingModes.set(id, "LAZY");
    }
}
exports.PlumTree = PlumTree;
// We have to store a bit of plumtree info the messaging mode - in the sync agent. right?
