"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PromiseEnroller = void 0;
const errors_js_1 = require("../util/errors.js");
const multi_deferred_js_1 = require("./multi_deferred.js");
class PromiseEnroller {
    constructor(allowRejectedPromises = false) {
        Object.defineProperty(this, "promises", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Set()
        });
        Object.defineProperty(this, "sealed", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: false
        });
        Object.defineProperty(this, "multiDeferred", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new multi_deferred_js_1.MultiDeferred()
        });
        Object.defineProperty(this, "allowRejectedPromises", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.allowRejectedPromises = allowRejectedPromises;
    }
    enrol(promise) {
        if (this.sealed) {
            throw new errors_js_1.EarthstarError("Tried to enrol a promise when enrolment was already sealed.");
        }
        this.promises.add(promise);
        promise.then(() => {
            this.checkAllDone();
        }).catch((err) => {
            if (this.allowRejectedPromises) {
                // Swallow the error.
                return;
            }
            throw err;
        });
    }
    checkAllDone() {
        if (!this.sealed) {
            return;
        }
        if (this.allowRejectedPromises) {
            Promise.allSettled(this.promises).then(() => {
                this.multiDeferred.resolve();
            });
        }
        else {
            Promise.all(this.promises).then(() => {
                this.multiDeferred.resolve();
            }).catch(() => {
                this.multiDeferred.reject();
            });
        }
    }
    seal() {
        if (this.sealed) {
            return;
        }
        this.sealed = true;
        this.checkAllDone();
    }
    isSealed() {
        return this.sealed;
    }
    isDone() {
        const promise = this.multiDeferred.getPromise();
        return promise;
    }
}
exports.PromiseEnroller = PromiseEnroller;
