"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Syncer = void 0;
const dntShim = __importStar(require("../../_dnt.shims.js"));
const crypto_js_1 = require("../crypto/crypto.js");
const util_js_1 = require("../formats/util.js");
const stream_utils_js_1 = require("../streams/stream_utils.js");
const misc_js_1 = require("../util/misc.js");
const sync_agent_js_1 = require("./sync_agent.js");
const transfer_manager_js_1 = require("./transfer_manager.js");
const multi_deferred_js_1 = require("./multi_deferred.js");
const deps_js_1 = require("../../deps.js");
const bumping_timeout_js_1 = require("./bumping_timeout.js");
const constants_js_1 = require("./constants.js");
/** Syncs the data of a Peer's replicas with that of another peer.
 *
 * ```ts
 * const syncer = peer.sync("https://my.server");
 *
 * syncer.onStatusChange((newStatus) => {
 *  console.log(newStatus);
 * });
 *
 * await syncer.isDone();
 *
 * console.log('Sync complete!');
 * ```
 */
class Syncer {
    constructor(opts) {
        Object.defineProperty(this, "peer", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "id", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: (0, misc_js_1.randomId)()
        });
        Object.defineProperty(this, "manager", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "partner", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "syncAgents", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Map()
        });
        Object.defineProperty(this, "syncAgentQueues", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Map()
        });
        Object.defineProperty(this, "appetite", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "statusBus", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new stream_utils_js_1.BlockingBus()
        });
        Object.defineProperty(this, "formats", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "transferManager", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "partnerIsFulfilled", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: (0, deps_js_1.deferred)()
        });
        Object.defineProperty(this, "isDoneMultiDeferred", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new multi_deferred_js_1.MultiDeferred()
        });
        Object.defineProperty(this, "heartbeatInterval", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        // Used to timeout when we haven't heard from the other peer in a while.
        Object.defineProperty(this, "bumpingTimeout", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        // Have to do this because we'll be using these values in a context where 'this' is different
        // (the streams below)
        this.manager = opts.manager;
        this.peer = opts.manager.peer;
        this.appetite = opts.partner.syncAppetite;
        this.formats = (0, util_js_1.getFormatsWithFallback)(opts.formats);
        this.partner = opts.partner;
        this.transferManager = new transfer_manager_js_1.TransferManager({
            partner: this.partner,
            formats: this.formats,
        });
        this.transferManager.onReportUpdate(async () => {
            await this.statusBus.send(this.getStatus());
        });
        this.heartbeatInterval = dntShim.setInterval(() => {
            this.partner.sendEvent({ kind: "HEARTBEAT" });
        }, 1000);
        (async () => {
            try {
                for await (const event of opts.partner.getEvents()) {
                    this.handleIncomingEvent(event);
                }
            }
            catch (err) {
                this.cancel(err || "Partner disconnected");
            }
        })();
        // Send off a salted handshake event
        const salt = (0, misc_js_1.randomId)();
        Promise.all(this.peer.shares().map((ws) => saltAndHashShare(salt, ws))).then((saltedShares) => {
            this.partner.sendEvent({
                kind: "DISCLOSE",
                salt,
                syncerId: this.id,
                shares: saltedShares,
                formats: this.formats
                    ? this.formats.map((f) => f.id)
                    : [util_js_1.DEFAULT_FORMAT.id],
                canRespond: false,
            });
        });
        this.transferManager.transfersRequestedByUsFinished().then(async () => {
            if (opts.partner.syncAppetite === "continuous") {
                return;
            }
            await this.partner.sendEvent({
                kind: "SYNCER_FULFILLED",
            });
            await this.partnerIsFulfilled;
            clearInterval(this.heartbeatInterval);
            await this.partner.closeConnection();
            this.bumpingTimeout.close();
            this.isDoneMultiDeferred.resolve();
        });
        this.peer.onReplicasChange(async (replicas) => {
            if (this.appetite === "once") {
                return;
            }
            const shares = Array.from(replicas.keys());
            for (const [address, agent] of this.syncAgents) {
                if (shares.includes(address) === false) {
                    await agent.cancel("Replica was removed from peer.");
                    this.syncAgents.delete(address);
                }
            }
            // send out disclose event again
            const salt = (0, misc_js_1.randomId)();
            Promise.all(shares.map((ws) => saltAndHashShare(salt, ws))).then((saltedShares) => {
                this.partner.sendEvent({
                    kind: "DISCLOSE",
                    salt,
                    syncerId: this.id,
                    shares: saltedShares,
                    formats: this.formats
                        ? this.formats.map((f) => f.id)
                        : [util_js_1.DEFAULT_FORMAT.id],
                    canRespond: true,
                });
            });
        });
        this.bumpingTimeout = new bumping_timeout_js_1.BumpingTimeout(() => {
            this.cancel("No communication from the other peer in the last three seconds.");
        }, constants_js_1.TIMEOUT_MS);
    }
    addShare(address, formats, initiateMessaging) {
        // Bail if we already have a sync agent for this share.
        if (this.syncAgents.has(address)) {
            return;
        }
        const replica = this.peer.getReplica(address);
        if (!replica) {
            console.error("Couldn't get the replica for a share we had in common.");
            return;
        }
        const agent = new sync_agent_js_1.SyncAgent({
            replica,
            formats,
            syncerManager: this.manager,
            transferManager: this.transferManager,
            initiateMessaging: initiateMessaging,
            payloadThreshold: this.partner.payloadThreshold,
            rangeDivision: this.partner.rangeDivision,
            syncAppetite: this.appetite,
        });
        agent.onStatusUpdate(() => {
            this.statusBus.send(this.getStatus());
        });
        this.syncAgents.set(address, agent);
        this.transferManager.registerSyncAgent(agent);
        const existingQueue = this.syncAgentQueues.get(address);
        let queueToUse;
        if (!existingQueue) {
            const queue = new deps_js_1.AsyncQueue();
            queueToUse = queue;
            this.syncAgentQueues.set(address, queue);
        }
        else {
            queueToUse = existingQueue;
        }
        (async () => {
            for await (const incomingEvent of queueToUse) {
                agent.sendEvent(incomingEvent);
            }
        })();
        (async () => {
            for await (const event of agent.events()) {
                this.partner.sendEvent({
                    to: address,
                    ...event,
                });
            }
        })();
    }
    /** Handle inbound events from the other peer. */
    async handleIncomingEvent(event) {
        this.bumpingTimeout.bump();
        switch (event.kind) {
            // Handle an incoming salted handsake
            case "DISCLOSE": {
                const intersectingFormats = (0, util_js_1.getFormatIntersection)(event.formats, this.formats);
                if (intersectingFormats.length === 0) {
                    break;
                }
                const serverSaltedSet = new Set(event.shares);
                const commonShareSet = new Set();
                const uncommonShareSet = new Set();
                // For each of our own shares, hash with the salt given to us by the event
                // If it matches any of the hashes sent by the other side, we have a share in common.
                for (const planeAddr of this.peer.shares()) {
                    const saltedAddr = await saltAndHashShare(event.salt, planeAddr);
                    if (serverSaltedSet.has(saltedAddr)) {
                        commonShareSet.add(planeAddr);
                    }
                    else {
                        uncommonShareSet.add(planeAddr);
                    }
                }
                const initiateMessaging = this.id > event.syncerId;
                for (const share of commonShareSet) {
                    this.addShare(share, intersectingFormats, initiateMessaging);
                }
                for (const share of uncommonShareSet) {
                    const syncAgent = this.syncAgents.get(share);
                    if (syncAgent) {
                        syncAgent.cancel("No longer in common with the other peer.");
                        this.syncAgents.delete(share);
                    }
                }
                this.transferManager.registerOtherSyncerId(event.syncerId);
                if (this.appetite === "once") {
                    this.transferManager.allSyncAgentsKnown();
                }
                if (event.canRespond) {
                    const salt = (0, misc_js_1.randomId)();
                    Promise.all(this.peer.shares().map((ws) => saltAndHashShare(salt, ws))).then((saltedShares) => {
                        this.partner.sendEvent({
                            kind: "DISCLOSE",
                            salt,
                            syncerId: this.id,
                            shares: saltedShares,
                            formats: this.formats
                                ? this.formats.map((f) => f.id)
                                : [util_js_1.DEFAULT_FORMAT.id],
                            canRespond: false,
                        });
                    });
                }
                if (commonShareSet.size === 0 && this.appetite === "once") {
                    this.partner.sendEvent({
                        "kind": "SYNCER_FULFILLED",
                    });
                }
                break;
            }
            case "SYNCER_FULFILLED": {
                this.partnerIsFulfilled.resolve();
                break;
            }
            case "HEARTBEAT": {
                break;
            }
            // Pass on to the sync agent.
            default: {
                // Get the share address;
                const { to } = event;
                const agentQueue = this.syncAgentQueues.get(to);
                if (!agentQueue) {
                    const queue = new deps_js_1.AsyncQueue();
                    queue.push(event);
                    this.syncAgentQueues.set(to, queue);
                    break;
                }
                agentQueue.push(event);
            }
        }
    }
    /** Get the status of all shares' syncing progress. */
    getStatus() {
        const status = {};
        for (const [shareAddr, agent] of this.syncAgents) {
            status[shareAddr] = {
                docs: agent.getStatus(),
                attachments: this.transferManager.getReport()[shareAddr] || [],
            };
        }
        return status;
    }
    /** Fires the provided callback whenever any shares' syncing progress changes. */
    onStatusChange(callback) {
        return this.statusBus.on(callback);
    }
    /** Stop syncing. */
    async cancel(reason) {
        this.bumpingTimeout.close();
        this.isDoneMultiDeferred.reject(reason);
        for (const [_addr, agent] of this.syncAgents) {
            await agent.cancel();
        }
        clearInterval(this.heartbeatInterval);
        this.transferManager.cancel();
        await this.partner.closeConnection();
    }
    // externally callable... to get a readable...
    /** Handle a transfer request which came out of band. This method is mostly used by other syncing APIs. */
    handleTransferRequest({ shareAddress, path, author, source, kind, formatName }) {
        if (this.isDoneMultiDeferred.state === "rejected") {
            return;
        }
        const replica = this.peer.getReplica(shareAddress);
        if (!replica) {
            return;
        }
        let counterpartId = "unused";
        if (kind === "upload") {
            // Someone is uploading to us...
            // Get their counterpart Id.
            const agent = this.syncAgents.get(shareAddress);
            if (agent) {
                counterpartId = agent.counterpartId;
            }
        }
        return this.transferManager.handleTransferRequest({
            replica,
            author,
            formatName,
            kind,
            path,
            source,
            counterpartId,
        });
    }
    /** If the syncer was configured with the `appetite: 'once'`, this promise will resolve when all the partner's existing documents and attachments have synchronised. */
    isDone() {
        return this.isDoneMultiDeferred.getPromise();
    }
}
exports.Syncer = Syncer;
function saltAndHashShare(salt, share) {
    return crypto_js_1.Crypto.sha256base32(salt + share + salt);
}
