"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SyncerManager = void 0;
const dntShim = __importStar(require("../../_dnt.shims.js"));
const deps_js_1 = require("../../deps.js");
const stream_utils_js_1 = require("../streams/stream_utils.js");
const errors_js_1 = require("../util/errors.js");
const doc_thumbnail_tree_js_1 = require("./doc_thumbnail_tree.js");
const plum_tree_js_1 = require("./plum_tree.js");
const syncer_js_1 = require("./syncer.js");
class SyncerManager {
    constructor(peer) {
        /** A map of syncer IDs to syncers  */
        Object.defineProperty(this, "syncers", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Map()
        });
        Object.defineProperty(this, "syncerEventBus", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new stream_utils_js_1.BlockingBus()
        });
        Object.defineProperty(this, "peer", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        // INITIAL SYNC (range-based set reconciliation)
        /** DocThumbnail  */
        Object.defineProperty(this, "docThumbnailTreeAndLookup", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Map()
        });
        /** We use this for creating doc thumbnails during tree generation. */
        Object.defineProperty(this, "hasher", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new deps_js_1.XXH64()
        });
        // CONTINUOUS SYNC
        Object.defineProperty(this, "plumTrees", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Map()
        });
        this.peer = peer;
    }
    addPartner(partner, description, formats) {
        // Add a new syncer with a given partner config.
        const syncer = new syncer_js_1.Syncer({
            manager: this,
            partner,
            formats,
        });
        this.syncers.set(syncer.id, { syncer, description });
        this.syncerEventBus.send(this.syncers);
        return syncer;
    }
    /** Returns a record of syncers with their given descriptions as keys. */
    getSyncers() {
        return this.syncers;
    }
    /** Create or retrieve an existing DocThumbnailTree for use with range-based reconciliation. */
    getDocThumbnailTreeAndDocLookup(share, formats) {
        // Create the key by which to look up information
        const formatNames = formats.map((f) => f.id);
        // e.g. "+something.a123 es.4,es.5"
        const key = `${share} ${formatNames}`;
        // Get a fingerprint tree for a given share
        const treeAndLookup = this.docThumbnailTreeAndLookup.get(key);
        // If it exists, just return it.
        if (treeAndLookup) {
            const [tree, lookup] = treeAndLookup;
            return { tree, lookup, treeIsReady: Promise.resolve(true) };
        }
        // If it doesn't, we need to create a tree.
        // Get the replica to build the tree with.
        const replica = this.peer.getReplica(share);
        if (!replica) {
            // This really shouldn't happen so I'm happy to throw here.
            throw new errors_js_1.EarthstarError("A DocThumbnailTree was requested for a share the peer doesn't know about, big problem!");
        }
        // We're going to need this. Deconstructing as we need to use it in a different this context.
        const hasher = this.hasher;
        // Make a stream of all docs of the given formats.
        const queryStream = replica.getQueryStream({
            historyMode: "all",
            orderBy: "localIndex ASC",
        }, "everything", formats);
        // Instantiate a new tree.
        const tree = new doc_thumbnail_tree_js_1.DocThumbnailTree();
        const lookup = {};
        const treeIsReady = (0, deps_js_1.deferred)();
        // Pipe the docs out, building the tree with them.
        queryStream.pipeTo(new dntShim.WritableStream({
            write(event) {
                // We've processed all existing items and the tree is ready to use.
                if (event.kind === "processed_all_existing") {
                    treeIsReady.resolve();
                    return;
                }
                // Create the doc thumbnail.
                // First create a hash of the path and author.
                hasher.reset();
                hasher.update(`${event.doc.path} ${event.doc.author}`);
                const pathAuthorHash = hasher.digest().toString(16);
                // Compbine with doc timestamp
                // e.g. "104342348 a83dfac89ac"
                const thumbnail = `${event.doc.timestamp} ${pathAuthorHash}`;
                if (event.kind === "existing" || event.kind === "success") {
                    // If the doc is existing or new, add it to the lookup.
                    lookup[pathAuthorHash] = [
                        event.doc.timestamp,
                        event.doc.path,
                        event.doc.author,
                    ];
                    tree.insert(thumbnail);
                }
                if (event.kind === "expire") {
                    // If it's expiring, remove it.
                    delete lookup[pathAuthorHash];
                    tree.remove(thumbnail);
                }
            },
        }));
        return { tree, lookup, treeIsReady };
    }
    getPlumTree(address) {
        const maybePlumTree = this.plumTrees.get(address);
        if (maybePlumTree) {
            return maybePlumTree;
        }
        const plumTree = new plum_tree_js_1.PlumTree();
        this.plumTrees.set(address, plumTree);
        return plumTree;
    }
    // Subscribe
    onSyncersChange(callback) {
        return this.syncerEventBus.on(callback);
    }
}
exports.SyncerManager = SyncerManager;
