"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.identifyBufOrBytes = exports.isBuffer = exports.isBytes = exports.b64StringToBytes = exports.concatBytes = exports.stringLengthInBytes = exports.stringToBytes = exports.bytesToString = void 0;
/**
 * This file provides common operations on Uint8Arrays.
 * It should not use any Buffers to do so.
 * Any function that uses Buffer should be in buffers.ts.
 * This helps us avoid bringing in the heavy polyfill for Buffer
 * when bundling for the browser.
 */
const dntShim = __importStar(require("../../_dnt.shims.js"));
const deps_js_1 = require("../../deps.js");
const decoder = new dntShim.TextDecoder();
const encoder = new dntShim.TextEncoder();
//--------------------------------------------------
function bytesToString(bytes) {
    return decoder.decode(bytes);
}
exports.bytesToString = bytesToString;
function stringToBytes(str) {
    return encoder.encode(str);
}
exports.stringToBytes = stringToBytes;
//--------------------------------------------------
function stringLengthInBytes(str) {
    // TODO: is there a more efficient way to do this?
    // If we had a Buffer we could just do Buffer.byteLength(str, 'utf-8');
    return stringToBytes(str).length;
}
exports.stringLengthInBytes = stringLengthInBytes;
function concatBytes(a, b) {
    // Checks for truthy values or empty arrays on each argument
    // to avoid the unnecessary construction of a new array and
    // the type comparison
    if (!b || b.length === 0)
        return a;
    if (!a || a.length === 0)
        return b;
    var c = new Uint8Array(a.length + b.length);
    c.set(a);
    c.set(b, a.length);
    return c;
}
exports.concatBytes = concatBytes;
//--------------------------------------------------
function b64StringToBytes(b64string) {
    return deps_js_1.rfc4648.base64.parse(b64string);
}
exports.b64StringToBytes = b64StringToBytes;
//--------------------------------------------------
function isBytes(bytes) {
    return bytes?.constructor?.name === "Uint8Array";
    //return bytes.writeUInt8 === undefined && bytes instanceof Uint8Array;
}
exports.isBytes = isBytes;
function isBuffer(buf) {
    // do this without any official reference to Buffer
    // to avoid bringing in the Buffer polyfill
    return buf?.constructor?.name === "Buffer";
    //buf instanceof Buffer;
}
exports.isBuffer = isBuffer;
function identifyBufOrBytes(bufOrBytes) {
    if (isBytes(bufOrBytes))
        return "bytes";
    if (isBuffer(bufOrBytes))
        return "buffer";
    return "?";
}
exports.identifyBufOrBytes = identifyBufOrBytes;
