"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.notErr = exports.isErr = exports.NotSupportedError = exports.NotImplementedError = exports.ConnectionRefusedError = exports.TimeoutError = exports.NetworkError = exports.ReplicaCacheIsClosedError = exports.ReplicaIsClosedError = exports.ValidationError = exports.EarthstarError = void 0;
/** Generic top-level error class that other Earthstar errors inherit from. */
class EarthstarError extends Error {
    constructor(message) {
        super(message || "");
        this.name = "EarthstarError";
    }
}
exports.EarthstarError = EarthstarError;
/** Validation failed on a document, share address, author address, etc. */
class ValidationError extends EarthstarError {
    constructor(message) {
        super(message || "Validation error");
        this.name = "ValidationError";
    }
}
exports.ValidationError = ValidationError;
/** An IReplica or IReplicaDriver was used after close() was called on it. */
class ReplicaIsClosedError extends EarthstarError {
    constructor(message) {
        super(message || "a Replica or ReplicaDriver was used after being closed");
        this.name = "ReplicaIsClosedError";
    }
}
exports.ReplicaIsClosedError = ReplicaIsClosedError;
/** An IReplica or IReplicaDriver was used after close() was called on it. */
class ReplicaCacheIsClosedError extends EarthstarError {
    constructor(message) {
        super(message || "a ReplicaCache was used after being closed");
        this.name = "ReplicaCacheIsClosedError";
    }
}
exports.ReplicaCacheIsClosedError = ReplicaCacheIsClosedError;
/** A server URL is bad or the network is down */
class NetworkError extends EarthstarError {
    constructor(message) {
        super(message || "network error");
        this.name = "NetworkError";
    }
}
exports.NetworkError = NetworkError;
class TimeoutError extends EarthstarError {
    constructor(message) {
        super(message || "timeout error");
        this.name = "TimeoutError";
    }
}
exports.TimeoutError = TimeoutError;
/** A server won't accept writes */
class ConnectionRefusedError extends EarthstarError {
    constructor(message) {
        super(message || "connection refused");
        this.name = "ConnectionRefused";
    }
}
exports.ConnectionRefusedError = ConnectionRefusedError;
class NotImplementedError extends EarthstarError {
    constructor(message) {
        super(message || "not implemented yet");
        this.name = "NotImplementedError";
    }
}
exports.NotImplementedError = NotImplementedError;
class NotSupportedError extends EarthstarError {
    constructor(message) {
        super(message || "Not supported");
        this.name = "NotSupportedError";
    }
}
exports.NotSupportedError = NotSupportedError;
/** Check if any value is a subclass of EarthstarError (return true) or not (return false) */
function isErr(x) {
    return x instanceof EarthstarError;
}
exports.isErr = isErr;
/** Check if any value is a subclass of EarthstarError (return false) or not (return true) */
function notErr(x) {
    return !(x instanceof EarthstarError);
}
exports.notErr = notErr;
