/** This module is browser compatible. */
import { BinarySearchTree } from "./binary_search_tree.js";
import { RedBlackNode } from "./red_black_node.js";
export * from "./_comparators.js";
/**
 * A red-black tree. This is a kind of self-balancing binary search tree.
 * The values are in ascending order by default,
 * using JavaScript's built in comparison operators to sort the values.
 */
export declare class RedBlackTree<T> extends BinarySearchTree<T> {
    protected root: RedBlackNode<T> | null;
    constructor(compare?: (a: T, b: T) => number);
    /** Creates a new red-black tree from an array like or iterable object. */
    static from<T>(collection: ArrayLike<T> | Iterable<T> | RedBlackTree<T>): RedBlackTree<T>;
    static from<T>(collection: ArrayLike<T> | Iterable<T> | RedBlackTree<T>, options: {
        Node?: typeof RedBlackNode;
        compare?: (a: T, b: T) => number;
    }): RedBlackTree<T>;
    static from<T, U, V>(collection: ArrayLike<T> | Iterable<T> | RedBlackTree<T>, options: {
        compare?: (a: U, b: U) => number;
        map: (value: T, index: number) => U;
        thisArg?: V;
    }): RedBlackTree<U>;
    protected removeFixup(parent: RedBlackNode<T> | null, current: RedBlackNode<T> | null): void;
    /**
     * Adds the value to the binary search tree if it does not already exist in it.
     * Returns true if successful.
     */
    insert(value: T): boolean;
    /**
     * Removes node value from the binary search tree if found.
     * Returns true if found and removed.
     */
    remove(value: T): boolean;
}
