import { RedBlackTree } from "../../../../std@0.158.0/collections/red_black_tree.js";
import { Direction, RedBlackNode } from "../../../../std@0.158.0/collections/red_black_node.js";
import { LiftingMonoid } from "../lifting_monoid.js";
/** A node for a FingerprintTree, augmented with a label and lifted value. Can update the labels of its ancestors. */
export declare class FingerprintNode<ValueType = string, LiftType = string> extends RedBlackNode<ValueType> {
    parent: FingerprintNode<ValueType, LiftType> | null;
    left: FingerprintNode<ValueType, LiftType> | null;
    right: FingerprintNode<ValueType, LiftType> | null;
    label: LiftType;
    liftedValue: LiftType;
    private monoid;
    constructor(parent: FingerprintNode<ValueType, LiftType> | null, value: ValueType, monoid: LiftingMonoid<ValueType, LiftType>);
    updateLabel(updateParent?: boolean, reason?: string): void;
}
type CombinedLabel<V, L> = [L, [number, [V[], V]]];
export type NodeType<V, L> = FingerprintNode<V, CombinedLabel<V, L>>;
/** A self-balancing tree which can return fingerprints for ranges of items it holds using a provided monoid. */
export declare class FingerprintTree<ValueType, LiftedType> extends RedBlackTree<ValueType> {
    protected root: NodeType<ValueType, LiftedType> | null;
    monoid: LiftingMonoid<ValueType, [
        LiftedType,
        [number, [ValueType[], ValueType]]
    ]>;
    private cachedMinNode;
    constructor(
    /** The lifting monoid which is used to label nodes and derive fingerprints from ranges. */
    monoid: LiftingMonoid<ValueType, LiftedType>, 
    /** A function to sort values by. Will use JavaScript's default comparison if not provided. */
    compare: (a: ValueType, b: ValueType) => number);
    /** Return the lowest value within this tree. Useful for constructing the maximum range of the tree, which will be [x, x) where x is the result of this function. */
    getLowestValue(): ValueType;
    rotateNode(node: NodeType<ValueType, LiftedType>, direction: Direction): void;
    removeFixup(parent: NodeType<ValueType, LiftedType> | null, current: NodeType<ValueType, LiftedType> | null): void;
    private insertFingerprintNode;
    /** Insert a value into the tree. Will create a lifted value for the resulting node, and update the labels of all rotated and parent nodes in the tree. */
    insert(value: ValueType): boolean;
    /** Remove a value frem the tree. Will recalculate labels for all rotated and parent nodes. */
    remove(value: ValueType): boolean;
    /** Calculates a fingerprint of items within the given range, inclusive of xx and exclusive of y. Also returns the size of the range, the items contained within it, */
    getFingerprint(x: ValueType, y: ValueType, nextTree?: NodeType<ValueType, LiftedType>): {
        /** The fingeprint of this range. */
        fingerprint: LiftedType;
        /** The size of the range. */
        size: number;
        /** The items within this range. */
        items: ValueType[];
        /** A tree to be used for a subsequent call of `getFingerprint`, where the given y param for the previous call is the x param for the next one. */
        nextTree: NodeType<ValueType, LiftedType> | null;
    };
    /** Find the first node holding a value greater than or equal to the given value. */
    private findGteNode;
    private aggregateUntil;
    private aggregateUp;
    private aggregateDown;
    isValueEqual(a: ValueType, b: ValueType): boolean;
}
export {};
