import { FingerprintTree } from "../fingerprint_tree/fingerprint_tree.js";
import { RangeMessengerConfig } from "./range_messenger_config.js";
type ProcessStageResult<V, L> = {
    type: "emptySet";
    canRespond: false;
} | {
    type: "lowerBound";
    value: V;
} | {
    type: "payload";
    payload: V;
    end?: {
        canRespond: boolean;
        upperBound: V;
    };
} | {
    type: "emptyPayload";
    upperBound: V;
} | {
    type: "fingerprint";
    fingerprint: L;
    upperBound: V;
} | {
    type: "done";
    upperBound: V;
} | {
    type: "terminal";
};
export type RangeMessengerOpts<EncodedMessageType, ValueType, LiftedType> = {
    tree: FingerprintTree<ValueType, LiftedType>;
    fingerprintEquals: (a: LiftedType, b: LiftedType) => boolean;
    encoding: RangeMessengerConfig<EncodedMessageType, ValueType, LiftedType>;
    /** If the size of a newly partitioned range is equal to or less than this number, that range's elements will be sent to the other peer. _Must_ be higher than 1.*/
    payloadThreshold: number;
    /** How many parts to subdivide ranges into. */
    rangeDivision: number;
};
/** Produces and responds to messages, enabling efficient reconciliation of two sets. */
export declare class RangeMessenger<EncodedMessageType, ValueType, LiftedType> {
    private tree;
    private encoding;
    private isDoneTee;
    private insertionCallbacks;
    private fingerprintEquals;
    private payloadThreshold;
    private rangeDivision;
    constructor(opts: RangeMessengerOpts<EncodedMessageType, ValueType, LiftedType>);
    /** The lower bound of the next message, derived from the upper bound of the previous call to respond.*/
    private lowerBoundFromPrev;
    /** Decodes an incoming message, and remembers its upper bound to use as a lower bound for the next message. */
    private decode;
    /** A collated payload object created from many previous payload messages.  */
    private collatedPayload;
    private collatePayloads;
    /** A tree returned by the previous call of tree.getFingeprint, to be used by the next call to `.respond`. */
    private reusableTree;
    /** A tiny convenience for casting the reusable tree to the right type. */
    private setReusableTree;
    /** Formulate the reply to a given message, as well as carry out any side effects such as inserting items into the tree. */
    private process;
    /** The last upper bound of a series of adjacent 'done' messages, used to consolidate multiple done messages into a single one. */
    private lastAdjacentDoneUpperBound;
    /** Consolidate many adjacent done messages into a single done message, with the upper bound of the last one. */
    private consolidateAdjacentDones;
    /** Indicates if we are done judging from received messages from the current round of messages. */
    private isDoneSoFar;
    /** Determine if reconciliation has been achieved by looking at messages. */
    private checkIsDone;
    encode(message: ProcessStageResult<ValueType, LiftedType>): EncodedMessageType;
    /** Formulates and returns the appropriate response given a message from another peer. May insert values into the RangeMessenger's tree.
     *
     * This method must be called with the messages from another peer, in the order they came.
     */
    respond(message: EncodedMessageType): Array<EncodedMessageType>;
    isDone(): import("../../../../std@0.158.0/async/deferred.js").Deferred<unknown>;
    /** Returns the opening messages for initiating an exchange between two peers. */
    initialMessages(partitionItems?: (items: ValueType[]) => ValueType[][]): Iterable<EncodedMessageType>;
    onInsertion(callback: (value: ValueType) => void): () => void;
}
export {};
