export type RangeMessengerConfig<EncodedType, ValueType, LiftType> = {
    encode: {
        emptySet: (canRespond: boolean) => EncodedType;
        lowerBound: (value: ValueType) => EncodedType;
        payload: (value: ValueType, end?: {
            canRespond: boolean;
            upperBound: ValueType;
        }) => EncodedType;
        emptyPayload: (upperBound: ValueType) => EncodedType;
        done: (upperBound: ValueType) => EncodedType;
        fingerprint: (fingerprint: LiftType, upperBound: ValueType) => EncodedType;
        terminal: () => EncodedType;
    };
    decode: {
        getType: (message: EncodedType) => "emptySet" | "lowerBound" | "payload" | "emptyPayload" | "done" | "fingerprint" | "terminal";
        emptySet: (message: EncodedType) => boolean;
        lowerBound: (message: EncodedType) => ValueType;
        payload: (message: EncodedType) => {
            value: ValueType;
            end?: {
                canRespond: boolean;
                upperBound: ValueType;
            };
        };
        /** Returns the upper bound of the message */
        emptyPayload: (message: EncodedType) => ValueType;
        /** Returns the upper bound of the message */
        done: (message: EncodedType) => ValueType;
        fingerprint: (message: EncodedType) => {
            fingerprint: LiftType;
            upperBound: ValueType;
        };
        terminal: (e: EncodedType) => true;
    };
};
type ObjEncoding<V, L> = {
    type: "emptySet";
    canRespond: boolean;
} | {
    type: "lowerBound";
    value: V;
} | {
    type: "payload";
    payload: V;
    end?: {
        canRespond: boolean;
        upperBound: V;
    };
} | {
    type: "emptyPayload";
    upperBound: V;
} | {
    type: "fingerprint";
    fingerprint: L;
    upperBound: V;
} | {
    type: "done";
    upperBound: V;
} | {
    type: "terminal";
};
export declare const objConfig: RangeMessengerConfig<ObjEncoding<string, string>, string, string>;
export {};
