import { AuthorAddress, AuthorShortname, Base32String, ParsedAddress, ShareAddress, ShareName } from "../util/doc-types.js";
import { ValidationError } from "../util/errors.js";
/** Put a short name and pub key together into an identity address. */
export declare function assembleAuthorAddress(name: AuthorShortname, encodedPubkey: Base32String): AuthorAddress;
/** Put a share name and encoded pub key together into a share address. */
export declare function assembleShareAddress(name: ShareName, encodedPubkey: Base32String): ShareAddress;
/** Check that an identity address is valid. */
export declare function checkAuthorIsValid(addr: AuthorAddress): true | ValidationError;
/** Check that a share address is valid. */
export declare function checkShareIsValid(addr: ShareAddress): true | ValidationError;
/** Parse a share or author address into its parts */
export declare function parseAuthorOrShareAddress(address: AuthorAddress | ShareAddress): ParsedAddress | ValidationError;
/** Parse an author address into its parts. */
export declare function parseAuthorAddress(address: AuthorAddress): ParsedAddress | ValidationError;
/** Parse a share address into its parts. */
export declare function parseShareAddress(address: ShareAddress): ParsedAddress | ValidationError;
interface ParseAddressOpts {
    sigil: string;
    separator: string;
    minNameLength: number;
    maxNameLength: number;
    minPubkeyLength: number;
    maxPubkeyLength: number;
    allowedNameCharacters: string;
    allowedPubkeyCharacters: string;
    pubkeyMustStartWithB: boolean;
}
export declare function parseAddress(address: string, opts: ParseAddressOpts): ParsedAddress | ValidationError;
export {};
