import { AuthorAddress, DocBase, DocInputBase, LocalIndex, Path, ShareAddress, Signature, Timestamp } from "../util/doc-types.js";
import { IFormat } from "./format_types.js";
/** Contains data written and signed by an identity. */
export interface DocEs4 extends DocBase<"es.4"> {
    /** Which document format the doc adheres to, e.g. `es.4`. */
    format: "es.4";
    author: AuthorAddress;
    content: string;
    contentHash: string;
    /** When the document should be deleted, as a UNIX timestamp in microseconds. */
    deleteAfter: number | null;
    path: Path;
    /** Used to verify the authorship of the document. */
    signature: Signature;
    /** When the document was written, as a UNIX timestamp in microseconds (millionths of a second, e.g. `Date.now() * 1000`).*/
    timestamp: Timestamp;
    /** The share this document is from.
     * Shares were previously called workspaces, but we didn't want to break compatibility with previous versions by renaming this field.
     */
    workspace: ShareAddress;
    _localIndex?: LocalIndex;
}
/** A partial es.4 doc that is about to get written. The rest of the properties will be computed automatically. */
export interface DocInputEs4 extends DocInputBase<"es.4"> {
    /** The format the document adheres to, e.g. `es.4` */
    format: "es.4";
    path: Path;
    content: string;
    /** A UNIX timestamp in microseconds indicating when the document was written. Determined automatically if omitted. */
    timestamp?: number;
    /** A UNIX timestamp in microseconds indicating when the document should be deleted by.*/
    deleteAfter?: number | null;
}
/**
 * Validator for the 'es.4' format. Checks if documents are spec-compliant before ingesting, and signs them according to spec.
 * @link https://earthstar-project.org/specs/data-spec-es4
 */
export declare const FormatEs4: IFormat<"es.4", DocInputEs4, DocEs4, undefined>;
