import * as dntShim from "../../_dnt.shims.js";
import { AuthorAddress, DocBase, DocInputBase, LocalIndex, Path, ShareAddress, Signature, Timestamp } from "../util/doc-types.js";
import { IFormat } from "./format_types.js";
/** Contains data written and signed by an identity. */
export interface DocEs5 extends DocBase<"es.5"> {
    /** Which document format the doc adheres to, e.g. `es.5`. */
    format: "es.5";
    /** The address of the author keypair which wrote this document. */
    author: AuthorAddress;
    /** Text content. If the document has an attachment, this should be a description of that attachment. */
    text: string;
    /** Base32 SHA256 hash of this document's text. */
    textHash: string;
    /** When the document should be deleted, as a UNIX timestamp in microseconds. */
    deleteAfter?: number;
    /** The path this document was written to. */
    path: Path;
    /** Used to verify the authorship of the document. */
    signature: Signature;
    /** Used to verify the author knows the share's secret */
    shareSignature: Signature;
    /** When the document was written, as a UNIX timestamp in microseconds (millionths of a second, e.g. `Date.now() * 1000`).*/
    timestamp: Timestamp;
    /** The public address of the share this document is from. */
    share: ShareAddress;
    /** The size of the associated attachment in bytes, if any. */
    attachmentSize?: number;
    /** The sha256 hash of the associated attachment, if any. */
    attachmentHash?: string;
    _localIndex?: LocalIndex;
}
/** A partial es.5 doc that is about to get written. The rest of the properties will be computed automatically. */
export interface DocInputEs5 extends DocInputBase<"es.5"> {
    /** The format the document adheres to, e.g. `es.5` */
    format: "es.5";
    path: Path;
    /** Can be left blank if there is a previous version of the document with an attachment. */
    text?: string;
    /** Data as Uint8Array or ReadableStream, to be used as document's associated attachment. */
    attachment?: Uint8Array | dntShim.ReadableStream<Uint8Array>;
    /** A UNIX timestamp in microseconds indicating when the document was written. Determined automatically if omitted. */
    timestamp?: number;
    /** A UNIX timestamp in microseconds indicating when the document should be deleted by.*/
    deleteAfter?: number | null;
}
export type ConfigEs5 = {
    shareSecret: string | undefined;
};
/**
 * Format for 'es.5' documents. Supports attachments and share keypairs.
 * @link https://earthstar-project.org/specs/data-spec-es5
 */
export declare const FormatEs5: IFormat<"es.5", DocInputEs5, DocEs5, ConfigEs5>;
