import { AuthorAddress, FormatName, Path, Timestamp } from "../util/doc-types.js";
/** Filters a query by document attributes. */
export interface QueryFilter {
    /** Match an exact path. */
    path?: Path;
    pathStartsWith?: string;
    pathEndsWith?: string;
    /** Match documents with a given author. */
    author?: AuthorAddress;
    timestamp?: Timestamp;
    /** Match documents newer than the given timestamp. */
    timestampGt?: Timestamp;
    /** Match documents older than the given timestamp. */
    timestampLt?: Timestamp;
}
/** Represents fetching all historical versions of a document by authors, or just the latest versions. */
export type HistoryMode = "latest" | "all";
/** Describes a query for fetching documents from a replica. */
export interface Query<FormatsType extends FormatName[]> {
    /** Whether to fetch all historical versions of a document or just the latest versions. */
    historyMode?: HistoryMode;
    /** The order to return docs in. Defaults to `path ASC`. */
    orderBy?: "path ASC" | "path DESC" | "localIndex ASC" | "localIndex DESC";
    /** Only fetch documents which come after a certain point. */
    startAfter?: {
        /** Only documents after this localIndex. Only works when ordering by localIndex. */
        localIndex?: number;
        /** Only documents after this path. Only works when ordering by path. */
        path?: string;
    };
    filter?: QueryFilter;
    /** The maximum number of documents to return. */
    limit?: number;
    formats?: FormatsType;
}
export declare const DEFAULT_QUERY: Query<["es.5"]>;
