import { DocBase, ShareAddress } from "../../util/doc-types.js";
import { DocDriverMemory } from "./memory.js";
/** A replica driver which perists to LocalStorage, which stores a maximum of five megabytes per domain. If you're storing multiple shares, this limit will be divided among all their replicas.
 * Works in browsers and Deno.
 */
export declare class DocDriverLocalStorage extends DocDriverMemory {
    _localStorageKeyConfig: string;
    _localStorageKeyDocs: string;
    /**
     * @param share - The address of the share the replica belongs to.
     * @param key - An optional key you can use to differentiate storage for the same share on the same device.
     */
    constructor(share: ShareAddress, key?: string);
    close(erase: boolean): Promise<void>;
    _getConfigSync(key: string): string | undefined;
    _setConfigSync(key: string, value: string): void;
    _listConfigKeysSync(): string[];
    _deleteConfigSync(key: string): boolean;
    getConfig(key: string): Promise<string | undefined>;
    setConfig(key: string, value: string): Promise<void>;
    listConfigKeys(): Promise<string[]>;
    deleteConfig(key: string): Promise<boolean>;
    upsert<FormatType extends string, DocType extends DocBase<FormatType>>(doc: DocType): Promise<DocType>;
}
