import { DocBase, LocalIndex, Path, ShareAddress } from "../../util/doc-types.js";
import { Query } from "../../query/query-types.js";
import { IReplicaDocDriver } from "../replica-types.js";
/** An in-memory replica driver. Its contents will be lost when it is closed.
 * Works everywhere.
 */
export declare class DocDriverMemory implements IReplicaDocDriver {
    share: ShareAddress;
    _maxLocalIndex: LocalIndex;
    _isClosed: boolean;
    _configKv: Record<string, string>;
    docByPathAndAuthor: Map<string, DocBase<string>>;
    docsByPathNewestFirst: Map<Path, DocBase<string>[]>;
    latestDocsByPath: Map<string, DocBase<string>>;
    /**
     * @param share - The address of the share the replica belongs to.
     */
    constructor(share: ShareAddress);
    isClosed(): boolean;
    close(erase: boolean): Promise<void>;
    getConfig(key: string): Promise<string | undefined>;
    setConfig(key: string, value: string): Promise<void>;
    listConfigKeys(): Promise<string[]>;
    deleteConfig(key: string): Promise<boolean>;
    getMaxLocalIndex(): Promise<number>;
    _getAllDocs(): Promise<DocBase<string>[]>;
    _getLatestDocs(): Promise<DocBase<string>[]>;
    queryDocs(queryToClean: Query<string[]>): Promise<DocBase<string>[]>;
    upsert<DocType extends DocBase<string>>(doc: DocType): Promise<DocType>;
    eraseExpiredDocs(): Promise<DocBase<string>[]>;
}
