import { ShareAddress } from "../../util/doc-types.js";
import { Query } from "../../query/query-types.js";
export interface ReplicaSqliteOptsCreate {
    mode: "create";
    share: ShareAddress;
    filename: string;
}
export interface ReplicaSqliteOptsOpen {
    mode: "open";
    share: ShareAddress | null;
    filename: string;
}
export interface ReplicaSqliteOptsCreateOrOpen {
    mode: "create-or-open";
    share: ShareAddress;
    filename: string;
}
export type ReplicaSqliteOpts = ReplicaSqliteOptsCreate | ReplicaSqliteOptsOpen | ReplicaSqliteOptsCreateOrOpen;
export declare const MAX_LOCAL_INDEX_QUERY = "SELECT MAX(localIndex) from docs;";
export declare const UPSERT_CONFIG_QUERY = "INSERT OR REPLACE INTO config (key, content) VALUES (:key, :content);";
export declare const SELECT_CONFIG_CONTENT_QUERY = "SELECT content FROM config WHERE key = :key;";
export declare const SELECT_KEY_CONFIG_QUERY = "SELECT key FROM config";
export declare const DELETE_CONFIG_QUERY = "DELETE FROM config WHERE key = :key;";
export declare const UPSERT_DOC_QUERY = "INSERT OR REPLACE INTO docs (doc, localIndex, pathAuthor)\nVALUES (:doc, :localIndex, :pathAuthor)";
export declare const SELECT_EXPIRED_DOC_QUERY = "SELECT doc FROM docs WHERE deleteAfter <= :now";
export declare const DELETE_EXPIRED_DOC_QUERY = "DELETE FROM docs WHERE deleteAfter <= :now";
export declare const SET_ENCODING_QUERY = "PRAGMA encoding = \"UTF-8\";";
export declare const GET_ENCODING_QUERY = "PRAGMA encoding;";
export declare const CREATE_DOCS_TABLE_QUERY = "CREATE TABLE IF NOT EXISTS docs (\n    doc TEXT NOT NULL,\n    pathAuthor TEXT NOT NULL,\n    localIndex NUMBER NOT NULL UNIQUE,\n    format TEXT GENERATED ALWAYS AS (json_extract(doc, '$.format')) VIRTUAL NOT NULL,\n    path TEXT GENERATED ALWAYS AS (json_extract(doc, '$.path')) VIRTUAL NOT NULL,\n    author TEXT GENERATED ALWAYS AS (json_extract(doc, '$.author')) VIRTUAL NOT NULL,\n    timestamp NUMBER GENERATED ALWAYS AS (json_extract(doc, '$.timestamp')) VIRTUAL NOT NULL,\n    signature TEXT GENERATED ALWAYS AS (json_extract(doc, '$.signature')) VIRTUAL NOT NULL,\n    deleteAfter NUMBER GENERATED ALWAYS AS (json_extract(doc, '$.deleteAfter')) VIRTUAL,\n\t\tPRIMARY KEY(pathAuthor)\n);";
export declare const CREATE_INDEXES_QUERY = "CREATE INDEX IF NOT EXISTS docsFormat ON docs(format); \n   CREATE INDEX IF NOT EXISTS docsPath ON docs(path);\n   CREATE INDEX IF NOT EXISTS docsAuthor ON docs(author);\n   CREATE INDEX IF NOT EXISTS docsTimestamp ON docs(timestamp);\n   CREATE INDEX IF NOT EXISTS docsSignature ON docs(signature);\n   CREATE INDEX IF NOT EXISTS docsDeleteAfter ON docs(deleteAfter);\n   CREATE INDEX IF NOT EXISTS docsLocalIndex ON docs(localIndex);\n   CREATE INDEX IF NOT EXISTS docsPathAuthor ON docs(pathAuthor);\n  ";
export declare const CREATE_CONFIG_TABLE_QUERY = "CREATE TABLE IF NOT EXISTS config (\n\t\tkey TEXT NOT NULL PRIMARY KEY,\n\t\tcontent TEXT NOT NULL\n);";
export declare function makeDocQuerySql(query: Query<string[]>, now: number, mode: "documents" | "delete"): {
    sql: string;
    params: Record<string, any>;
};
