/// <reference types="node" />
import { createServer } from "node:http";
import { IServerExtension } from "./extensions/extension.js";
export type ServerOpts = {
    port?: number;
    server: ReturnType<typeof createServer>;
};
/**
 * An extensible server able to synchronise with other peers.
 *
 * A server's functionality can be extended using extensions of type `IServerExtension`.
 */
export declare class Server {
    private core;
    private server;
    /**
     * Create a new server with an array of extensions.
     * @param extensions - The extensions used by the server. Extensions will be registered in the order you provide them in, as one extension may depend on the actions of another. For example, the `ExtensionServeContent` may rely on a replica created by `ExtensionKnownShares`.
     */
    constructor(extensions: IServerExtension[], opts: ServerOpts);
    close(): void;
}
