/// <reference types="ws" />
/// <reference types="node" />
import * as dntShim from "../../_dnt.shims.js";
export declare class CombineStream<T> {
    private closed;
    private transform;
    private writer;
    readable: dntShim.ReadableStream<T>;
    private writables;
    private checkClosed;
    getWritableStream(): dntShim.WritableStream<T>;
    close(): Promise<void>;
    isClosed(): boolean;
}
export declare class CloneStream<ChunkType> {
    private transformStream;
    private sourceReadable;
    writable: dntShim.WritableStream<ChunkType>;
    getReadableStream(): dntShim.ReadableStream<ChunkType>;
}
export declare class CloneMidStream<ChunkType> {
    private closed;
    private subscribers;
    private writables;
    private checkClosed;
    writable: dntShim.WritableStream<ChunkType>;
    constructor();
    getReadableStream(): dntShim.ReadableStream<ChunkType>;
    close(): Promise<void>;
    isClosed(): boolean;
}
export declare class MultiStream<T> {
    private closed;
    private combineStream;
    private cloneStream;
    private checkClosed;
    constructor(joinMidStream?: boolean);
    getWritableStream(): dntShim.WritableStream<T>;
    getReadableStream(): dntShim.ReadableStream<T>;
    close(): void;
    isClosed(): boolean;
}
export declare class CallbackSink<T> implements dntShim.UnderlyingSink<T> {
    private callbacks;
    onWrite(callback: (chunk: T) => void | Promise<void>): () => void;
    write(chunk: T): Promise<void>;
    close(): void;
}
export declare class BlockingBus<T> {
    private callbacks;
    private lock;
    on(cb: (item: T) => void | Promise<void>): () => void;
    send(item: T): Promise<void>;
}
export declare class PassThroughTransformer<ChunkType> implements Transformer<ChunkType, ChunkType> {
    transform(chunk: ChunkType, controller: dntShim.TransformStreamDefaultController<ChunkType>): void;
}
export declare class LockStream {
    private doneEnroller;
    private writable;
    private writer;
    private closed;
    private checkClosed;
    run(cb: () => void | Promise<void>): Promise<void>;
    close(): Promise<void>;
    isClosed(): boolean;
}
export type OrCh<Ch extends string> = "*" | Ch;
export type Channelled<ChannelType extends string, KeyType extends string> = {
    [Key in KeyType]: OrCh<ChannelType>;
};
export declare class ChannelTransformer<ChannelType extends string, KeyType extends string, ItemType extends Channelled<ChannelType, KeyType>> implements Transformer<ItemType, ItemType> {
    private channel;
    private channelKey;
    constructor(channel: ChannelType, key: KeyType);
    transform(chunk: ItemType, controller: dntShim.TransformStreamDefaultController<ItemType>): void;
}
export declare class ChannelMultiStream<ChannelType extends string, KeyType extends string, ItemType extends Channelled<ChannelType, KeyType>> {
    private multistream;
    private channelKey;
    constructor(key: KeyType, joinMidStream?: boolean);
    getWritableStream(): dntShim.WritableStream<ItemType>;
    getReadableStream(channel: OrCh<ChannelType>): dntShim.ReadableStream<ItemType>;
}
export declare class StreamSplitter<ChunkType> {
    private transforms;
    private incomingCloner;
    writable: dntShim.WritableStream<ChunkType>;
    private getKey;
    constructor(getKey: (chunk: ChunkType) => string | undefined);
    getReadable(key: string): dntShim.ReadableStream<ChunkType>;
}
export declare function websocketWritable<T>(socketOrUrl: dntShim.WebSocket | string, prepareToSend: (outgoing: T) => string | ArrayBufferLike | dntShim.Blob | ArrayBufferView): dntShim.WritableStream<T>;
export declare function websocketReadable<T>(socketOrUrl: dntShim.WebSocket | string, prepareForQueue: (event: MessageEvent) => T): dntShim.ReadableStream<T>;
