import * as dntShim from "../../_dnt.shims.js";
import { FormatsArg } from "../formats/format_types.js";
import { IPeer } from "../peer/peer-types.js";
import { NotSupportedError } from "../util/errors.js";
import { Syncer } from "./syncer.js";
import { GetTransferOpts, ISyncPartner, SyncAppetite, SyncerEvent } from "./syncer_types.js";
/** A syncing partner to be used with local instances of `IPeer`.
 * Works everywhere.
 */
export declare class PartnerLocal<FormatsType, IncomingTransferSourceType extends undefined> implements ISyncPartner<undefined> {
    concurrentTransfers: number;
    payloadThreshold: number;
    rangeDivision: number;
    syncAppetite: SyncAppetite;
    private outgoingQueue;
    private incomingQueue;
    private partnerPeer;
    partnerSyncer: Syncer<IncomingTransferSourceType, FormatsType>;
    /**
     * @param peer - The target peer to sync with.
     * @param peerSelf - Our own peer.
     */
    constructor(peer: IPeer, peerSelf: IPeer, appetite: SyncAppetite, formats?: FormatsArg<FormatsType>);
    getEvents(): AsyncIterable<SyncerEvent>;
    sendEvent(event: SyncerEvent): Promise<void>;
    closeConnection(): Promise<void>;
    getDownload(opts: GetTransferOpts): Promise<dntShim.ReadableStream<Uint8Array> | undefined>;
    handleUploadRequest(_opts: GetTransferOpts): Promise<dntShim.WritableStream<Uint8Array> | NotSupportedError>;
    handleTransferRequest(_source: IncomingTransferSourceType, _kind: "upload" | "download"): Promise<dntShim.ReadableStream<Uint8Array> | dntShim.WritableStream<Uint8Array> | undefined | NotSupportedError>;
}
