import { DocThumbnail, SyncAgentDocEvent, SyncAgentHaveEvent } from "./syncer_types.js";
/** A push-lazy-push-multicast tree, or 'PlumTree'. Organises a network of interconnected peers into a spanning tree where messaging resiliency, latency, and bandwidth are optimised.
 *
 * When Earthstar sync agents finish their initial reconciliation phase they are switched to a mode where they are managed by a plumtree.
 */
export declare class PlumTree {
    private messagingModes;
    getMode(id: string): "EAGER" | "LAZY";
    /** A list of previously received message IDs, used to check incoming messages for duplicates.
     */
    private eagerMessageThumbnails;
    /** A map of DocThumbnails to timers. */
    private haveTimeouts;
    /** Triggered when the other peer sends a DOC message. Returns a boolean indicating whether to send a PRUNE event to the peer we got this message from. */
    onEagerMessage(id: string, event: SyncAgentDocEvent): boolean;
    /** Triggered when the other peer sends a HAVE message */
    onLazyMessage(event: SyncAgentHaveEvent, dispatchGraft: (thumbnail: DocThumbnail) => void): void;
    /** Triggered when the other peer sends a graft, i.e. WANT message. */
    onGraftMessage(id: string): void;
    /** Triggered when the other peer sends a PRUNE message. */
    onPrune(id: string): void;
}
