import { DocThumbnail, SyncAgentEvent, SyncAgentOpts, SyncAgentStatus, SyncAppetite } from "./syncer_types.js";
import { FormatsArg } from "../formats/format_types.js";
import { Replica } from "../replica/replica.js";
import { AsyncQueue, Deferred } from "../../deps.js";
import { SyncerManager } from "./syncer_manager.js";
import { TransferManager } from "./transfer_manager.js";
import { MultiformatReplica } from "../replica/multiformat_replica.js";
/** Mediates synchronisation on behalf of a `Replica`.
 */
export declare class SyncAgent<F> {
    /** A bus we can update the SyncAgent's status from, and which others can subscribe to. */
    private statusBus;
    /** A multi deferred describing if th the SyncAgent has finished or not. */
    private isDoneMultiDeferred;
    /** Messages coming in from the other peer */
    private inboundEventQueue;
    /** Messages generated from us destined for the other peer. */
    private outboundEventQueue;
    private gossiperInboundQueue;
    private reconcilerInboundQueue;
    private hasPrepared;
    private hasReconciled;
    private hasCheckedAllExistingDocsForAttachments;
    private requestedCount;
    private sentDocsCount;
    private receivedDocsCount;
    /** An internal ID we use to distinguish messages from the agent we're syncing with from other messages and docs. */
    counterpartId: string;
    sendEvent(event: SyncAgentEvent): void;
    events(): AsyncIterable<SyncAgentEvent>;
    replica: Replica;
    /** The current status of this SyncAgent */
    getStatus(): SyncAgentStatus;
    constructor(opts: SyncAgentOpts<F>);
    /** Subscribe to status updates with a callback. */
    onStatusUpdate(callback: (status: SyncAgentStatus) => void): () => void;
    /** Signal the SyncAgent to wrap up syncing early. */
    cancel(reason?: string): Promise<void>;
    isDone(): Deferred<void>;
}
type GossiperOpts<F> = {
    inboundEventQueue: AsyncQueue<SyncAgentEvent>;
    outboundEventQueue: AsyncQueue<SyncAgentEvent>;
    syncerManager: SyncerManager;
    formats: FormatsArg<F> | undefined;
    replica: MultiformatReplica;
    counterpartId: string;
    transferManager: TransferManager<F, unknown>;
    wantTracker: WantTracker;
    reconciliationIsDone: Deferred<number>;
    syncAppetite: SyncAppetite;
    cancel: () => Promise<void>;
    onDocReceived: () => Promise<void>;
    onDocSent: () => Promise<void>;
    onDocRequested: () => Promise<void>;
};
export declare class SyncAgentGossiper<F> {
    private isPartnerFulfilled;
    private isFulfilled;
    constructor(opts: GossiperOpts<F>);
    /** A promise which completes when this gossiper has gotten everything it has asked for and its counterpart is fulfilled.
     *
     * Will never resolve if sync appetite is 'continuous'.
     */
    get isDone(): Promise<[unknown, unknown]>;
}
/** Tracks which documents a SyncAgent has sent WANT messages for and how many have actually been received. */
declare class WantTracker {
    private isSealed;
    private wantedThumbnails;
    private enroller;
    private received;
    id: string;
    addWantedThumbnail(thumbnail: DocThumbnail): void;
    receivedWantedThumbnail(thumbnail: DocThumbnail): void;
    isRequested(thumbnail: DocThumbnail): boolean;
    isReceived(thumbnail: DocThumbnail): boolean;
    seal(): void;
    isDone(): Deferred<unknown>;
    get requestedCount(): number;
    get receivedCount(): number;
}
export {};
