import { DefaultFormats } from "../formats/format_types.js";
import { IPeer } from "../peer/peer-types.js";
import { AuthorAddress, Path } from "../util/doc-types.js";
import { SyncerOpts, SyncerStatus } from "./syncer_types.js";
/** Syncs the data of a Peer's replicas with that of another peer.
 *
 * ```ts
 * const syncer = peer.sync("https://my.server");
 *
 * syncer.onStatusChange((newStatus) => {
 *  console.log(newStatus);
 * });
 *
 * await syncer.isDone();
 *
 * console.log('Sync complete!');
 * ```
 */
export declare class Syncer<IncomingTransferSourceType, FormatsType = DefaultFormats> {
    peer: IPeer;
    id: string;
    private manager;
    private partner;
    private syncAgents;
    private syncAgentQueues;
    private appetite;
    private statusBus;
    private formats;
    private transferManager;
    private partnerIsFulfilled;
    private isDoneMultiDeferred;
    private heartbeatInterval;
    private bumpingTimeout;
    constructor(opts: SyncerOpts<FormatsType, IncomingTransferSourceType>);
    private addShare;
    /** Handle inbound events from the other peer. */
    private handleIncomingEvent;
    /** Get the status of all shares' syncing progress. */
    getStatus(): SyncerStatus;
    /** Fires the provided callback whenever any shares' syncing progress changes. */
    onStatusChange(callback: (status: SyncerStatus) => void): () => void;
    /** Stop syncing. */
    cancel(reason?: Error | string): Promise<void>;
    /** Handle a transfer request which came out of band. This method is mostly used by other syncing APIs. */
    handleTransferRequest({ shareAddress, path, author, source, kind, formatName }: {
        shareAddress: string;
        formatName: string;
        path: Path;
        author: AuthorAddress;
        source: IncomingTransferSourceType;
        kind: "upload" | "download";
    }): Promise<void> | undefined;
    /** If the syncer was configured with the `appetite: 'once'`, this promise will resolve when all the partner's existing documents and attachments have synchronised. */
    isDone(): import("../../deps.js").Deferred<unknown>;
}
