import { FormatsArg } from "../formats/format_types.js";
import { IPeer } from "../peer/peer-types.js";
import { AuthorAddress, Path, ShareAddress, Timestamp } from "../util/doc-types.js";
import { DocThumbnailTree } from "./doc_thumbnail_tree.js";
import { PlumTree } from "./plum_tree.js";
import { Syncer } from "./syncer.js";
import { ISyncPartner } from "./syncer_types.js";
/** A thumbnail pointing to a timestamp, path, and author address. */
type DocThumbnailHashToDocLookup = Record<string, [
    Timestamp,
    Path,
    AuthorAddress
]>;
export declare class SyncerManager {
    /** A map of syncer IDs to syncers  */
    private syncers;
    private syncerEventBus;
    peer: IPeer;
    constructor(peer: IPeer);
    addPartner<I, F>(partner: ISyncPartner<I>, description: string, formats?: FormatsArg<F>): Syncer<I, F>;
    /** Returns a record of syncers with their given descriptions as keys. */
    getSyncers(): Map<string, {
        description: string;
        syncer: Syncer<unknown, unknown>;
    }>;
    /** DocThumbnail  */
    private docThumbnailTreeAndLookup;
    /** We use this for creating doc thumbnails during tree generation. */
    private hasher;
    /** Create or retrieve an existing DocThumbnailTree for use with range-based reconciliation. */
    getDocThumbnailTreeAndDocLookup<F>(share: ShareAddress, formats: FormatsArg<F>): {
        tree: DocThumbnailTree;
        lookup: DocThumbnailHashToDocLookup;
        treeIsReady: Promise<true>;
    };
    private plumTrees;
    getPlumTree(address: ShareAddress): PlumTree;
    onSyncersChange(callback: (map: Map<string, {
        description: string;
        syncer: Syncer<unknown, unknown>;
    }>) => void | Promise<void>): () => void;
}
export {};
