import { Deferred } from "../../deps.js";
import { FormatDocType, FormatsArg } from "../formats/format_types.js";
import { Replica } from "../replica/replica.js";
import { AuthorAddress, Path, ShareAddress } from "../util/doc-types.js";
import { NotSupportedError } from "../util/errors.js";
import { AttachmentTransferReport, GetTransferOpts, ISyncPartner } from "./syncer_types.js";
import { SyncAgent } from "./sync_agent.js";
export type TransferManagerOpts<FormatsType, IncomingAttachmentSourceType> = {
    partner: ISyncPartner<IncomingAttachmentSourceType>;
    formats: FormatsArg<FormatsType>;
};
export declare class TransferManager<FormatsType, IncomingAttachmentSourceType> {
    private partner;
    private queue;
    private formats;
    private otherSyncerId;
    private receivedAllExpectedTransfersEnroller;
    /** A kind of promise which resolves when all attachment requests have been made. */
    private madeAllAttachmentRequestsEnroller;
    private formatsLookup;
    private reportDidUpdateBus;
    private expectedTransferPromises;
    constructor(opts: TransferManagerOpts<FormatsType, IncomingAttachmentSourceType>);
    registerSyncAgent(agent: SyncAgent<FormatsType>): void;
    allSyncAgentsKnown(): void;
    registerExpectedTransfer(share: ShareAddress, hash: string): void;
    private queueTransfer;
    handleDownload(doc: FormatDocType<FormatsType>, replica: Replica, counterpartId: string): Promise<NotSupportedError | "no_attachment" | "queued">;
    handleUpload(transferOpts: Omit<GetTransferOpts, "syncerId">, replica: Replica): Promise<boolean>;
    handleTransferRequest({ replica, path, author, source, kind, formatName, counterpartId }: {
        replica: Replica;
        formatName: string;
        path: Path;
        author: AuthorAddress;
        source: IncomingAttachmentSourceType;
        kind: "upload" | "download";
        counterpartId: string;
    }): Promise<void>;
    cancel(): void;
    getReport(): Record<string, AttachmentTransferReport[]>;
    onReportUpdate(cb: () => void): () => void;
    transfersRequestedByUsFinished(): Deferred<unknown>;
    registerOtherSyncerId(id: string): void;
    isAlreadyQueued(hash: string, kind: "upload" | "download"): boolean;
}
