/** Generic top-level error class that other Earthstar errors inherit from. */
export declare class EarthstarError extends Error {
    constructor(message?: string);
}
/** Validation failed on a document, share address, author address, etc. */
export declare class ValidationError extends EarthstarError {
    constructor(message?: string);
}
/** An IReplica or IReplicaDriver was used after close() was called on it. */
export declare class ReplicaIsClosedError extends EarthstarError {
    constructor(message?: string);
}
/** An IReplica or IReplicaDriver was used after close() was called on it. */
export declare class ReplicaCacheIsClosedError extends EarthstarError {
    constructor(message?: string);
}
/** A server URL is bad or the network is down */
export declare class NetworkError extends EarthstarError {
    constructor(message?: string);
}
export declare class TimeoutError extends EarthstarError {
    constructor(message?: string);
}
/** A server won't accept writes */
export declare class ConnectionRefusedError extends EarthstarError {
    constructor(message?: string);
}
export declare class NotImplementedError extends EarthstarError {
    constructor(message?: string);
}
export declare class NotSupportedError extends EarthstarError {
    constructor(message?: string);
}
/** Check if any value is a subclass of EarthstarError (return true) or not (return false) */
export declare function isErr<T>(x: T | Error): x is EarthstarError;
/** Check if any value is a subclass of EarthstarError (return false) or not (return true) */
export declare function notErr<T>(x: T | Error): x is T;
