/**
 * Logs are assigned a priority number.
 * Higher numbers are less important information.
 * Set your desired log level higher to get more info.
 *
 *  -1 nothing
 *   0 only errors
 *   1 also warnings
 *   2 also logs
 *   3 also debugs
 *
 * Logs also come from different "sources" which can
 * have different log level settings.
 *
 * Two ways to modify the log level settings:
 *
 * 1. Set an environment variable in the shell.
 *    This applies to all "sources" (can't be set individually)
 *         EARTHSTAR_LOG_LEVEL=2 npm run test
 *
 * 2. Use setLogLevels() to globally modify the levels:
 *         setLogLevels({ sync: 2 });
 *
 * The environment variable wins over the numbers set by setLogLevels.
 */
type LogSource = string;
export declare enum LogLevel {
    None = -1,
    Error = 0,
    Warn = 1,
    Log = 2,
    Info = 3,
    Debug = 4
}
export declare const DEFAULT_LOG_LEVEL = LogLevel.Error;
type LogLevels = Record<LogSource, LogLevel>;
export declare function updateLogLevels(newLogLevels: LogLevels): void;
export declare function setLogLevel(source: LogSource, level: LogLevel): void;
export declare function setDefaultLogLevel(level: LogLevel): void;
export declare function getLogLevel(source: LogSource): LogLevel;
export declare function getLogLevels(): LogLevels;
type LogColor = "blue" | "aqua" | "darkcyan" | "cyan" | "dimgray" | "slategray" | "green" | "springgreen" | "grey" | "darkMagenta" | "magenta" | "red" | "orangeRed" | "salmon" | "lightsalmon" | "gold" | "yellow";
export declare class Logger {
    source: LogSource;
    color: LogColor | undefined;
    constructor(source: LogSource, color?: LogColor);
    _print(level: LogLevel, showTag: boolean, indent: string, ...args: any[]): void;
    error(...args: any[]): void;
    warn(...args: any[]): void;
    log(...args: any[]): void;
    info(...args: any[]): void;
    debug(...args: any[]): void;
    blank(): void;
}
export {};
